__author__ = 'mahajrod'
import datetime
from collections import OrderedDict
import numpy as np
#import matplotlib

import pandas as pd

#matplotlib.use('Agg')
#os.environ['MPLCONFIGDIR'] = '/tmp/'

import matplotlib.pyplot as plt
plt.ioff()
from matplotlib.patches import Rectangle, Circle
from matplotlib.lines import Line2D
from matplotlib.collections import LineCollection

from BCBio import GFF

from Bio import AlignIO

from RouToolPa.Collections.General import SynDict
from RouToolPa.Routines.Matplotlib import MatplotlibRoutines
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Parsers.DESeq2 import CollectionPWC


class DrawingRoutines(MatplotlibRoutines, SequenceRoutines):
    def __init__(self):
        MatplotlibRoutines.__init__(self)

    @staticmethod
    def millions(x, pos):
        return '%1.1fMbp' % (x*1e-6)

    @staticmethod
    def billions(x, pos):
        return '%1.1fGbp' % (x*1e-9)

    def draw_chromosomes_with_features_simple(self, chromosomes_gff, genes_gff, output_prefix, figsize=(10, 10),
                                              sense_feature_color="green", antisense_feature_color="red",
                                              chromosome_color="black", label_fontsize=15,
                                              chromosome_name_field=None,
                                              ext_list=("png",), dpi=None,
                                              alias_fields_in_gff=("ID", "Name", "gene", "Alias"),
                                              id_field_in_gff="ID", deseq2_pwc_file=None, upregulated_color="green",
                                              downregulated_color="red", absent_expression_data_color="black",
                                              coloring_mode="strand"):
        if deseq2_pwc_file and (coloring_mode == "expression"):
            deseq2_pwc_collection = CollectionPWC(from_file=True, pwc_file=deseq2_pwc_file)

        figure = plt.figure(figsize=figsize, dpi=dpi)
        """
        if dpi:
            figure = plt.figure(figsize=figsize, dpi=dpi)
        else:
            figure = plt.figure(figsize=figsize)
        """
        subplot = plt.subplot(1, 1, 1)

        subplot.get_yaxis().set_visible(False)
        #subplot.get_xaxis().set_visible(False)
        #axes.xaxis.set_major_formatter(x_formatter)

        #subplot.spines['bottom'].set_color('none')
        subplot.spines['right'].set_color('none')
        subplot.spines['left'].set_color('none')
        subplot.spines['top'].set_color('none')

        chr_dict = OrderedDict()

        max_chr_len = 0
        chr_number = 0
        with open(chromosomes_gff, "r") as chr_fd:
            for line in chr_fd:
                if line[0] == "#":
                    continue
                line_list = line.split("\t")
                feature_scaffold = line_list[0]
                feature_type = line_list[2]
                feature_start = line_list[3]
                feature_end = line_list[4]
                feature_description_list = line_list[8].split(";")

                chr_name = None
                if chromosome_name_field:

                    for entry in feature_description_list:
                        entry_list = entry.split("=")
                        if entry_list[0] == chromosome_name_field:
                            chr_name = entry_list[1]

                if feature_type == "chromosome" or feature_type == "plasmid" or feature_type == "region":
                    chr_dict[feature_scaffold] = OrderedDict()
                    chr_dict[feature_scaffold]["Name"] = chr_name if chr_name else feature_scaffold
                    chr_dict[feature_scaffold]["Genes"] = OrderedDict()
                    chr_dict[feature_scaffold]["Length"] = int(feature_end)
                    if chr_dict[feature_scaffold]["Length"] > max_chr_len:
                        max_chr_len = chr_dict[feature_scaffold]["Length"]
                        chr_number += 1
                elif feature_type == "centromere":
                    # add check for case when centromere line is first in gff file

                    chr_dict[feature_scaffold]["Centromere"] = [int(feature_start), int(feature_end)]
        #print chr_dict
        if genes_gff:
            with open(genes_gff, "r") as gene_fd:
                for line in gene_fd:
                    if line[0] == "#":
                        continue
                    line_list = line.strip().split("\t")
                    feature_scaffold = line_list[0]
                    feature_type = line_list[2]
                    feature_start = line_list[3]
                    feature_end = line_list[4]
                    feature_strand = line_list[6]
                    description_list = line_list[-1].split(";")

                    entry_id = None
                    alias_list = []
                    #print "AAAAAAAAAAA"
                    #print line
                    for entry in description_list:
                        if entry[:len(id_field_in_gff)+1] == ("%s=" % id_field_in_gff):
                            entry_id = entry[len(id_field_in_gff)+1:]
                            alias_list.append(entry_id)
                        for alias_id in alias_fields_in_gff:
                            if entry[:len(alias_id)+1] == ("%s=" % alias_id):
                                alias_list += entry.split("=")[1].split(",")
                    #print entry_id
                    if not entry_id:
                        continue

                    chr_dict[feature_scaffold]["Genes"][entry_id] = [int(feature_start), int(feature_end), feature_strand, alias_list]

        #print chr_dict
        centromere_radius = int(max_chr_len/100)
        distance_between_chromosome_and_gene = centromere_radius * 2
        distance_between_chromosomes = centromere_radius * 8
        chromosome_width = int(centromere_radius / 2)
        gene_width = chromosome_width
        chromosome_position = - int(distance_between_chromosomes / 3)

        text_x_offset = -max_chr_len/15
        for chromosome in chr_dict:
            print("Drawing chromosome %s" % chr_dict[chromosome]["Name"])
            chromosome_position += distance_between_chromosomes
            chromosome_fragments_list = []
            if "Centromere" not in chr_dict[chromosome]:
                chromosome_fragments_list.append(Rectangle((1, chromosome_position), chr_dict[chromosome]["Length"],
                                                           chromosome_width, fill=True, edgecolor=chromosome_color,
                                                           facecolor=chromosome_color))
            else:
                # left arm of chromosome
                #print "Centromere"
                #print "Left arm"
                #print (1, chromosome_position), chr_dict[chromosome]["Centromere"][0], chromosome_width


                chromosome_fragments_list.append(Rectangle((1, chromosome_position), chr_dict[chromosome]["Centromere"][0],
                                                           chromosome_width, fill=True, edgecolor=chromosome_color,
                                                           facecolor=chromosome_color))
                chromosome_fragments_list.append(Circle((chr_dict[chromosome]["Centromere"][0] + centromere_radius,
                                                         chromosome_position + chromosome_width/2), centromere_radius,
                                                        fill=False, edgecolor=chromosome_color,
                                                        facecolor=chromosome_color))
                chromosome_fragments_list.append(Rectangle((chr_dict[chromosome]["Centromere"][0] + 2 * centromere_radius,
                                                            chromosome_position), chr_dict[chromosome]["Length"] - chr_dict[chromosome]["Centromere"][1],
                                                           chromosome_width, fill=True, edgecolor=chromosome_color,
                                                           facecolor=chromosome_color))
            for patch in chromosome_fragments_list:
                subplot.add_patch(patch)

            subplot.annotate(chr_dict[chromosome]["Name"], xy=(text_x_offset, chromosome_position),
                             xycoords='data', fontsize=label_fontsize,
                             ha='center', va='center')

            if chr_dict[chromosome]["Genes"]:
                for gene in chr_dict[chromosome]["Genes"]:
                    print("Adding feature %s: %i-%i, %s. Aliases: %s" % (gene, chr_dict[chromosome]["Genes"][gene][0],
                                                                         chr_dict[chromosome]["Genes"][gene][1],
                                                                         chr_dict[chromosome]["Genes"][gene][2],
                                                                         ",".join(chr_dict[chromosome]["Genes"][gene][3])))
                    gene_start = chr_dict[chromosome]["Genes"][gene][0]
                    if "Centromere" in chr_dict[chromosome]:
                        if gene_start >= chr_dict[chromosome]["Centromere"][1]:
                            gene_start += centromere_radius * 2

                    if deseq2_pwc_file and (coloring_mode == "expression"):
                        for alias in chr_dict[chromosome]["Genes"][gene][3]:
                            if alias in deseq2_pwc_collection:
                                gene_log2foldchange = deseq2_pwc_collection[alias].log2foldchange
                                break
                        else:
                            gene_log2foldchange = None
                        gene_color = absent_expression_data_color if gene_log2foldchange is None else upregulated_color if gene_log2foldchange > 0 else downregulated_color
                    else:
                        gene_color = sense_feature_color if chr_dict[chromosome]["Genes"][gene][2] == "+" else antisense_feature_color

                    gene_patch = Rectangle((gene_start, chromosome_position + (1 if chr_dict[chromosome]["Genes"][gene][2] == "+" else -1) * distance_between_chromosome_and_gene),
                                           chr_dict[chromosome]["Genes"][gene][1] - chr_dict[chromosome]["Genes"][gene][0] + 1,
                                           gene_width, fill=True, edgecolor=gene_color,
                                           facecolor=gene_color)
                    subplot.add_patch(gene_patch)
        plt.xlim(xmax=max_chr_len, xmin=text_x_offset)
        plt.ylim(ymax=chromosome_position+distance_between_chromosomes)
        plt.subplots_adjust(right=0.95)#bottom=0.1, right=0.8, top=0.9)
        for extension in ext_list:
            plt.savefig("%s.%s" % (output_prefix, extension), dpi=dpi)

    def draw_alignment(self, alignment, features, output_prefix, record_style=None, ext_list=["svg", "png"],
                       label_fontsize=13, left_offset=0.2, figure_width=8, id_synonym_dict=None,
                       id_replacement_mode="partial", domain_style="vlines"):
        """
        id_replacement_mode have to be either partial or exact
        """
        #from Routines import SequenceRoutines
        sequence_number = len(alignment)
        alignment_length = len(alignment[0].seq)

        figure = plt.figure(figsize=(figure_width, sequence_number))
        subplot = plt.subplot(1, 1, 1)

        subplot.get_yaxis().set_visible(False)
        #subplot.get_xaxis().set_visible(False)
        #axes.xaxis.set_major_formatter(x_formatter)

        #subplot.spines['bottom'].set_color('none')
        subplot.spines['right'].set_color('none')
        subplot.spines['left'].set_color('none')
        subplot.spines['top'].set_color('none')

        protein_height = 10

        dist_between_proteins = 10
        start_x = 0
        start_y = - dist_between_proteins

        gap_line_y_shift = int(protein_height/2)
        gap_line_y_jump = int(protein_height/2)

        domen_colors = []
        for feature in features:
            if (feature.type == "domen") or (feature.type == "domain"):
                domen_colors.append(subplot._get_lines.color_cycle.next())

        for record in alignment:

            gap_coords_list, gap_len_list = self.find_homopolymers(record.seq, "-", min_size=1,
                                                                   search_type="perfect")
            #print gap_coords_list, gap_len_list

            start_y += protein_height + dist_between_proteins
            gap_y_start = gap_line_y_shift + start_y
            gap_y_jump = gap_y_start + gap_line_y_jump
            prev_x = 0
            """
            figure.text(0, start_y, record.id, rotation=0, fontweight="bold", transform=subplot.transAxes, fontsize=9,
                         horizontalalignment='center',
                         verticalalignment='center')
            """
            if id_synonym_dict:
                if id_replacement_mode == "exact":
                    if record.id in id_synonym_dict:
                        record_label = id_synonym_dict[record.id]
                    else:
                        record_label = record.id
                        print("WARNING!!! Synonym for %s was not found" % record.id)
                elif id_replacement_mode == "partial":

                    partial_syn_list = []
                    for partial_syn in id_synonym_dict:
                        if partial_syn in record.id:
                            partial_syn_list.append(partial_syn)

                    if len(partial_syn_list) > 1:
                        print("WARNING!!! More than one possible replacement for %s was found: %s. No replacement then." % (record.id, ",".join(partial_syn_list)))
                        record_label = record.id
                    elif not partial_syn_list:
                        record_label = record.id
                        print("WARNING!!! Synonym for %s was not found" % record.id)
                    else:
                        record_label = id_synonym_dict[partial_syn_list[0]]
                else:
                    raise ValueError("Unknown id replacement mode")

            else:
                record_label = record.id

            subplot.annotate(record_label, xy=(0, gap_y_start), xycoords='data', fontsize=16,
                             xytext=(-15, 1.5 * gap_line_y_shift), textcoords='offset points',
                             ha='right', va='top')

            for gap_coords, gap_len in zip(gap_coords_list, gap_len_list):

                if gap_coords[0] != 0:

                    fragment = Rectangle((prev_x, start_y), gap_coords[0] - prev_x, protein_height, fill=False,
                                         edgecolor="black", facecolor="grey")
                    #print prev_x
                    #print gap_coords[0] - prev_x

                    subplot.add_patch(fragment)
                prev_x = gap_coords[1]
                #print [gap_coords[0], gap_coords[0] + int(gap_len/2) + 1, gap_coords[1]]
                plt.plot([gap_coords[0], gap_coords[0] + int(gap_len/2) + 1, gap_coords[1]], #plt.plot([gap_coords[0] + 2, gap_coords[0] + int(gap_len/2) + 1, gap_coords[1] - 1],
                         [gap_y_start, gap_y_jump, gap_y_start], color="black", linewidth=1)

            if not gap_coords_list:
                fragment = Rectangle((prev_x, start_y), alignment_length, protein_height, fill=False,
                                     edgecolor="black", facecolor="grey")
                subplot.add_patch(fragment)
            else:
                if gap_coords_list[-1][-1] != alignment_length:
                    fragment = Rectangle((prev_x, start_y), alignment_length - prev_x, protein_height, fill=False,
                                         edgecolor="black", facecolor="grey")
                    #print prev_x, alignment_length - prev_x
                    subplot.add_patch(fragment)
            i = 0
            for feature in features:
                if (feature.type == "domen") or (feature.type == "domain"):
                    #print feature.id, feature.location
                    if domain_style == "rectangle":
                        fragment = Rectangle((feature.location.start, start_y), len(feature)-1, protein_height, fill=False,
                                             facecolor="grey", edgecolor=domen_colors[i]) #edgecolor="green",
                        subplot.add_patch(fragment)
                    elif domain_style == "vlines":
                        plt.vlines(feature.location.start, protein_height, start_y + protein_height, colors=domen_colors[i])
                        plt.vlines(feature.location.end - 1, protein_height, start_y + protein_height, colors=domen_colors[i])
                    i += 1

        for feature in features:
            if feature.type == "domen":
                #print feature.id, feature.location
                subplot.annotate(feature.id, xy=(feature.location.start + len(feature)/2, gap_y_start + protein_height),
                                 xycoords='data', fontsize=label_fontsize,
                                 xytext=(0, 1.5 * gap_line_y_shift), textcoords='offset points', ha='center', va='top')

        plt.xlim(xmin=0, xmax=alignment_length + 10)
        plt.ylim(ymin=0, ymax=start_y + 2 * protein_height)
        #plt.tight_layout()
        plt.subplots_adjust(left=left_offset, right=0.95)#bottom=0.1, right=0.8, top=0.9)
        for extension in ext_list:
            plt.savefig("%s.%s" % (output_prefix, extension))

    def draw_alignment_from_file(self, alignment_file, feature_gff, output_prefix, alignment_style=None,
                                 alignment_format="fasta", ext_list=["svg", "png"], label_fontsize=13,
                                 left_offset=0.2, figure_width=8, id_synonym_dict=None,
                                 id_replacement_mode="partial"):

        alignment = AlignIO.read(alignment_file, format=alignment_format)
        if feature_gff:
            with open(feature_gff, "r") as gff_fd:
                record = list(GFF.parse(gff_fd))[0]
                features = record.features
                record_id = record.id
                gap_coords_list, gap_len_list = self.find_homopolymers(record.seq, "-", min_size=1,
                                                                       search_type="perfect")
        else:
            features = []

        self.draw_alignment(alignment, features, output_prefix, ext_list=ext_list, label_fontsize=label_fontsize,
                            left_offset=left_offset, figure_width=figure_width, id_synonym_dict=id_synonym_dict,
                            id_replacement_mode=id_replacement_mode)

    @staticmethod
    def draw_length_histogram(sequence_dict, output_prefix, number_of_bins=None, width_of_bins=None,
                              min_length=1, max_length=None, extensions=("png", "svg"),
                              legend_location='best'):
        length_dict = SynDict()

        for record in sequence_dict:
            length_dict[record] = len(sequence_dict[record].seq)

        length_dict.write("%s.len" % output_prefix)

        lengths = length_dict.values()

        max_len = max(lengths)
        min_len = min(lengths)
        median = np.median(lengths)
        mean = np.mean(lengths)

        if max_length is None:
            maximum_length = max_len
        else:
            maximum_length = max_length

        filtered = []

        if (maximum_length < max_len) and (min_length > 1):
            for entry in lengths:
                if min_length <= entry <= maximum_length:
                    filtered.append(entry)
        elif min_length > 1:
            for entry in lengths:
                if min_length <= entry:
                    filtered.append(entry)
        elif maximum_length < max_len:
            for entry in lengths:
                if entry <= maximum_length:
                    filtered.append(entry)
        else:
            filtered = lengths

        plt.figure(1, figsize=(6, 6))
        plt.subplot(1, 1, 1)

        if number_of_bins:
            bins = number_of_bins
        elif width_of_bins:
            bins = np.arange(min_length - 1, maximum_length, width_of_bins, dtype=np.int32)
            bins[0] += 1
            bins = np.append(bins, [maximum_length])
        else:
            bins = 30
        plt.hist(filtered, bins=bins)
        plt.xlim(xmin=min_length, xmax=maximum_length)
        plt.xlabel("Length")
        plt.ylabel("N")
        plt.title("Distribution of sequence lengths")
        plt.legend(("Min: %i\nMax: %i\nMean: %i\nMedian: %i" % (min_len, max_len, mean, median),), loc=legend_location)
        for ext in extensions:
            plt.savefig("%s.%s" % (output_prefix, ext))

    def draw_heatmap_and_three_percent_histograms(self, first_histo_values, second_histo_values,
                                                  third_histo_values, output_prefix, figsize=(12, 12),
                                                  extensions=("png", "svg"), stats_as_legend=True):
        """
        second_histo_values and third_histo_values are used to build heatmap
        """

        plt.figure(1, figsize=figsize)

        for (index, histo_values, title) in zip([1, 2, 3],
                                                [first_histo_values, second_histo_values, third_histo_values],
                                                ["Total support", "CDS support", "Intron support"]):
            subplot = plt.subplot(2, 2, index)

            self.percent_histogram(histo_values, output_prefix=None, n_bins=20, title=title, xlabel="%",
                                   ylabel="Number of transcripts", label=None, extensions=("png", "svg"),
                                   legend=None, legend_location="best", input_mode="percent", xmax=None,
                                   xmin=None, stats_as_legend=stats_as_legend)

        bins = np.linspace(0, 100, 21)

        subplot = plt.subplot(2, 2, 4)
        #print bins
        counts, xedges, yedges, image = plt.hist2d(second_histo_values,
                                                   third_histo_values,
                                                   bins=(bins, bins),
                                                   range=[[0, 100], [0, 100]])
        max_counts = int(np.nanmax(counts))

        cmap = plt.get_cmap('jet', max_counts)
        #cmap.set_under('gray')
        mappable = plt.cm.ScalarMappable(cmap=cmap)
        mappable.set_array([])
        mappable.set_clim(0.00001, max_counts)
        #mappable.set_array([])
        #mappable.set_clim(-0.5, ncolors+0.5)
        colorbar = plt.colorbar(mappable)
        plt.xlabel("CDS support")
        plt.ylabel("Intron support")
        plt.title("Transcript support")
        plt.tight_layout()
        for ext in extensions:
            plt.savefig("%s.%s" % (output_prefix, ext))

    @staticmethod
    def draw_precalculated_heatmap(heatmap_array, output_prefix=None, figsize=(5, 5), extensions=("png", "svg")):

        if output_prefix:
            figure = plt.figure(1, figsize=figsize)

        heatmap = plt.imshow(heatmap_array, origin='low', interpolation='none')

        if output_prefix:
            for ext in extensions:
                plt.savefig("%s.%s" % (output_prefix, ext))

    def draw_histogram_from_multiple_files(self, filelist, output_prefix,  filelabels=None, nbins=30,
                                           figsize=(5, 5), title=None, xlabel=None, ylabel=None,
                                           extensions=("png", "svg"), separator="\n", xmin=None, xmax=None,
                                           histtype="stepfilled"):

        colors = ["aqua", "blue", "black", "brown",
                  "red", "fuchsia", "maroon", "orange",
                  "purple", "yellow", "sienna", "green",
                  "olive"]

        if filelabels:
            labels = filelabels
        else:
            labels = []
            for filename in filelist:
                path, basename, extention = self.split_filename(filename)
                labels.append(basename)

        if (xmin is not None) and (xmax is not None):
            bins = np.linspace(xmin, xmax, nbins + 1)
            bin_middles = []
            for i in range(0, len(bins) - 1):
                bin_middles.append((bins[i] + bins[i+1])/2)
        else:
            bins = nbins

        data = []
        for filename, label in zip(filelist, labels):
            filedata = np.fromfile(filename, sep=separator)
            data.append(filedata)

            print("%s\tmin %f\t max %f" % (label, np.min(filedata), np.max(filedata)))

            # a = np.histogram(filedata, bins=bins)
            #print a

        #print xmin
        #print xmax
        #print bins

        figure = plt.figure(1, figsize=figsize)
        ax = plt.axes()
        ax.set_color_cycle(colors)

        #colors = plt.cm.jet(np.linspace(0.2, 1, len(filelist)))

        plt.hist(data, label=labels, bins=bins, histtype=histtype) #, color=colors)

        if xlabel:
            plt.xlabel(xlabel)
        if ylabel:
            plt.ylabel(ylabel)
        if title:
            plt.title(title)

        plt.xlim(xmin=xmin, xmax=xmax)
        plt.legend(loc="best")

        for ext in extensions:
            plt.savefig("%s.%s" % (output_prefix, ext))

    def test_colormap(self, colormap_name, color_number=10, output_prefix=None, extension_list=("png",), side=10):
        cmap = plt.get_cmap(colormap_name, color_number)
        figure = plt.figure(figsize=(4, 4))
        subplot = plt.subplot(1, 1, 1)
        subplot.get_yaxis().set_visible(False)
        subplot.get_xaxis().set_visible(False)

        posx = 0
        posy = 0
        for i in range(0, color_number):
            fragment = Rectangle((posx, posy), side, side, fill=True, edgecolor="black",
                                 facecolor=cmap(i), linewidth=0.5)
            posy += side
            subplot.add_patch(fragment)
        plt.ylim(ymin=0, ymax=posy)
        plt.title("Colormap %s" % colormap_name)

        if output_prefix:
            for ext in extension_list:
                plt.savefig("%s.%s" % (output_prefix, ext))
        else:
            plt.show()

    def draw_dot_plot_from_last_alignment(self, last_collection,
                                          output_prefix=None,
                                          extension_list=("png", ),
                                          target_black_list=(), target_white_list=(),
                                          target_ordered_list=(), target_reverse_list=(),
                                          query_black_list=(), query_white_list=(),
                                          query_ordered_list=(), query_reverse_list=(),
                                          figsize=(16, 16), dpi=300,
                                          grid_color='black',
                                          bar_color='grey',
                                          same_strand_color='red',
                                          diff_strand_color='blue',
                                          title=None,
                                          target_label=None,
                                          query_label=None,
                                          gridwidth=1,
                                          show_grid=True,
                                          linewidth=0.01,
                                          scaffold_label_fontsize=13,
                                          axes_label_fontstyle="italic",
                                          axes_label_weight="normal",
                                          axes_label_fontsize=15,
                                          axes_label_distance=6,
                                          antialiased_lines=None,
                                          target_scaffold_labels_angle=45,
                                          query_scaffold_labels_angle=0,
                                          show_target_labels=True,
                                          show_query_labels=True,
                                          bottom_offset=0.1,
                                          top_offset=0.9,
                                          left_offset=0.1,
                                          right_offset=0.9,
                                          x_axis_visible=False,
                                          y_axis_visible=False):

        target_scaffold_list = self.get_filtered_scaffold_list(last_collection.target_scaffold_list,
                                                               scaffold_black_list=target_black_list,
                                                               sort_scaffolds=False,
                                                               scaffold_ordered_list=target_ordered_list,
                                                               scaffold_white_list=target_white_list)

        query_scaffold_list = self.get_filtered_scaffold_list(last_collection.query_scaffold_list,
                                                              scaffold_black_list=query_black_list,
                                                              sort_scaffolds=False,
                                                              scaffold_ordered_list=query_ordered_list,
                                                              scaffold_white_list=query_white_list)

        target_length_df = last_collection.target_scaffold_lengths.loc[target_scaffold_list]
        target_length_df["cum_end"] = target_length_df["length"].cumsum()
        target_length_df["cum_start"] = target_length_df["cum_end"] - target_length_df["length"]

        total_target_len = target_length_df["length"].sum()

        query_length_df = last_collection.query_scaffold_lengths.loc[query_scaffold_list]
        query_length_df["cum_end"] = query_length_df["length"].cumsum()
        query_length_df["cum_start"] = query_length_df["cum_end"] - query_length_df["length"]

        total_query_len = query_length_df["length"].sum()

        bar_width_fraction = 0.02
        bar_width = int(max(total_query_len, total_target_len) * bar_width_fraction)

        print("%s\tDrawing..." % str(datetime.datetime.now()))
        print("%s\t\tInitializing figure..." % str(datetime.datetime.now()))

        figure = plt.figure(figsize=figsize, dpi=dpi)
        ax = plt.subplot(1, 1, 1)

        print("%s\t\tInitializing figure finished..." % str(datetime.datetime.now()))
        print("%s\t\tDrawing grid..." % str(datetime.datetime.now()))

        ax.add_patch(Rectangle((0, total_query_len), total_target_len, bar_width, color=bar_color))  # top bar
        ax.add_patch(Rectangle((0, -bar_width), total_target_len, bar_width, color=bar_color))       # bottom bar
        ax.add_patch(Rectangle((-bar_width, 0), bar_width, total_query_len, color=bar_color))        # left bar
        ax.add_patch(Rectangle((total_target_len, 0), bar_width, total_query_len, color=bar_color))  # right bar

        if show_grid:
            for query_cum_start in query_length_df["cum_start"]:
                ax.add_line(Line2D((-bar_width, total_target_len+bar_width), (query_cum_start, query_cum_start),
                                   color=grid_color, linewidth=gridwidth))

            ax.add_line(Line2D((-bar_width, total_target_len+bar_width), (total_query_len, total_query_len),
                               color=grid_color, linewidth=gridwidth))

            for target_cum_start in target_length_df["cum_start"]:
                ax.add_line(Line2D((target_cum_start, target_cum_start), (-bar_width, total_query_len + bar_width),
                                   color=grid_color, linewidth=gridwidth))
            ax.add_line(Line2D((total_target_len, total_target_len), (-bar_width, total_query_len + bar_width),
                               color=grid_color, linewidth=gridwidth))
        else:
            for query_cum_start in query_length_df["cum_start"]:
                ax.add_line(Line2D((-bar_width, 0), (query_cum_start, query_cum_start),
                                   color=grid_color, linewidth=gridwidth))
                ax.add_line(Line2D((total_target_len, total_target_len + bar_width), (query_cum_start, query_cum_start),
                                   color=grid_color, linewidth=gridwidth))

            ax.add_line(Line2D((-bar_width, 0), (total_query_len, total_query_len),
                               color=grid_color, linewidth=gridwidth))
            ax.add_line(Line2D((total_target_len, total_target_len + bar_width), (total_query_len, total_query_len),
                               color=grid_color, linewidth=gridwidth))

            for target_cum_start in target_length_df["cum_start"]:
                ax.add_line(Line2D((target_cum_start, target_cum_start), (-bar_width, 0),
                                   color=grid_color, linewidth=gridwidth))
                ax.add_line(Line2D((target_cum_start, target_cum_start), (total_query_len, total_query_len + bar_width),
                                   color=grid_color, linewidth=gridwidth))

            ax.add_line(Line2D((total_target_len, total_target_len), (-bar_width, 0),
                               color=grid_color, linewidth=gridwidth))
            ax.add_line(Line2D((total_target_len, total_target_len), (total_query_len, total_query_len + bar_width),
                               color=grid_color, linewidth=gridwidth))

        print("%s\t\tDrawing grid finished..." % str(datetime.datetime.now()))
        print("%s\t\tAdding labels..." % str(datetime.datetime.now()))

        if show_target_labels:
            for target_scaffold_id in target_scaffold_list:
                ax.text((target_length_df.loc[target_scaffold_id]["cum_start"] + target_length_df.loc[target_scaffold_id]["cum_end"])/2,
                        total_query_len + 1.5 * bar_width, target_scaffold_id, fontsize=scaffold_label_fontsize,
                        rotation=target_scaffold_labels_angle,
                        horizontalalignment='left',
                        verticalalignment='bottom',)
                ax.text((target_length_df.loc[target_scaffold_id]["cum_start"] + target_length_df.loc[target_scaffold_id]["cum_end"])/2,
                        -1.5 * bar_width, target_scaffold_id, fontsize=scaffold_label_fontsize,
                        rotation=target_scaffold_labels_angle,
                        horizontalalignment='right',
                        verticalalignment='top',)
        if show_query_labels:
            for query_scaffold_id in query_scaffold_list:
                ax.text(total_target_len + 1.5 * bar_width,
                        (query_length_df.loc[query_scaffold_id]["cum_start"] + query_length_df.loc[query_scaffold_id]["cum_end"])/2,
                         query_scaffold_id, fontsize=scaffold_label_fontsize,
                        rotation=query_scaffold_labels_angle,
                        horizontalalignment='left',
                        verticalalignment='center' if query_scaffold_labels_angle == 0 else 'bottom')
                ax.text(-1.5 * bar_width,
                        (query_length_df.loc[query_scaffold_id]["cum_start"] + query_length_df.loc[query_scaffold_id]["cum_end"])/2,
                         query_scaffold_id, fontsize=scaffold_label_fontsize,
                        rotation=query_scaffold_labels_angle,
                        horizontalalignment='right',
                        verticalalignment='center' if query_scaffold_labels_angle == 0 else 'top')

        if title:
            plt.title(title)
        if target_label:
            ax.text(total_target_len/2,
                    -bar_width*axes_label_distance,
                    target_label,
                    fontsize=axes_label_fontsize,
                    fontstyle=axes_label_fontstyle,
                    fontweight=axes_label_weight,
                    horizontalalignment='center',
                    verticalalignment='center')
        if query_label:
            ax.text(-bar_width*axes_label_distance,
                    total_query_len/2,
                    query_label,
                    fontsize=axes_label_fontsize,
                    fontstyle=axes_label_fontstyle,
                    fontweight=axes_label_weight,
                    rotation=90,
                    horizontalalignment='center',
                    verticalalignment='center')

        plt.xlim(xmin=-bar_width * 2, xmax=total_target_len + 2 * bar_width)
        plt.ylim(ymin=-bar_width * 2, ymax=total_query_len + 2 * bar_width)

        if not x_axis_visible:
            ax.spines['bottom'].set_color('none')
            ax.spines['top'].set_color('none')

        if not y_axis_visible:
            ax.spines['right'].set_color('none')
            ax.spines['left'].set_color('none')

        ax.get_yaxis().set_visible(y_axis_visible)
        ax.get_xaxis().set_visible(x_axis_visible)

        print("%s\t\tAdding labels finished..." % str(datetime.datetime.now()))
        print("%s\t\tDrawing alignments..." % str(datetime.datetime.now()))

        def line_segments_generator(dataframe):
            for row_tuple in dataframe.itertuples(index=False):
                yield (row_tuple[:2], row_tuple[2:])
        for query_scaffold_id in query_scaffold_list:
            for target_scaffold_id in target_scaffold_list:
                same_strand_records = \
                    last_collection.records[last_collection.records["target_id"].isin([target_scaffold_id])
                                            & last_collection.records["query_id"].isin([query_scaffold_id])
                                            & (last_collection.records["query_strand"] == last_collection.records["target_strand"])]

                if not same_strand_records.empty:
                    data = pd.DataFrame()

                    data["x1"] = same_strand_records["target_start"] + target_length_df.loc[target_scaffold_id]["cum_start"]
                    data["y1"] = same_strand_records["query_start"] + query_length_df.loc[query_scaffold_id]["cum_start"]

                    data["x2"] = data["x1"] + same_strand_records["target_hit_len"] - 1
                    data["y2"] = data["y1"] + same_strand_records["query_hit_len"] - 1

                    lines = LineCollection(line_segments_generator(data), colors=same_strand_color, linestyle='solid',
                                           linewidths=linewidth, antialiased=antialiased_lines)

                    ax.add_collection(lines)
                
                diff_strand_records = \
                    last_collection.records[last_collection.records["target_id"].isin([target_scaffold_id])
                                            & last_collection.records["query_id"].isin([query_scaffold_id])
                                            & (last_collection.records["query_strand"] != last_collection.records["target_strand"])]

                if not diff_strand_records.empty:

                    data = pd.DataFrame()
                    data["x1"] = diff_strand_records["target_start"] + target_length_df.loc[target_scaffold_id]["cum_start"]
                    data["x2"] = data["x1"] + diff_strand_records["target_hit_len"] - 1

                    data["y1"] = query_length_df.loc[query_scaffold_id]["length"] - diff_strand_records["query_start"] + query_length_df.loc[query_scaffold_id]["cum_start"]
                    data["y2"] = data["y1"] - diff_strand_records["query_hit_len"] + 1
                    data = data[["x1", "y1", "x2", "y2"]]

                    lines = LineCollection(line_segments_generator(data), colors=diff_strand_color, linestyle='solid',
                                           linewidths=linewidth, antialiased=antialiased_lines)

                    ax.add_collection(lines)

        print("%s\t\tDrawng alignments finished..." % str(datetime.datetime.now()))
        print("%s\tDrawing finished..." % str(datetime.datetime.now()))

        plt.subplots_adjust(left=left_offset, bottom=bottom_offset, right=right_offset, top=top_offset)

        if output_prefix:
            print("%s\tWriting to file..." % str(datetime.datetime.now()))
            for extension in extension_list:
                plt.savefig("%s.%s" % (output_prefix, extension))

            print("%s\tWriting to file finished..." % str(datetime.datetime.now()))



__author__ = 'mahajrod'


from RouToolPa.Tools.Abstract import Tool


class Tophat(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "tophat", path=path, max_threads=max_threads)

    def align(self, index, left_reads, right_reads=None, out_dir="tophat_output", read_mismatches=None,
              read_gap_length=None, read_edit_dist=None, read_realign_edit_dist=None, min_anchor=None,
              splice_mismatches=None, min_intron_length=None, max_intron_length=None, max_multihits=None,
              transcriptome_max_hits=None, max_insertion_length=None, max_deletion_length=None, quality=None,
              library_type=None, known_transcripts_file=None, transcriptome_index=None, raw_juncs_file=None,
              insertions_file=None, deletions_file=None, mate_inner_dist=None, mate_std_dev=None, no_novel_juncs=None,
              no_novel_indels=None, no_gtf_juncs=None, no_coverage_search=None, coverage_search=None,
              microexon_search=None, keep_tmp=None, tmp_dir=None, report_secondary_alignments=None, no_discordant=None,
              no_mixed=None, segment_mismatches=None, segment_length=None, bowtie_n=None, min_coverage_intron=None,
              max_coverage_intron=None, min_segment_intron=None, max_segment_intron=None, no_sort_bam=None,
              no_convert_bam=None, keep_fasta_order=None, allow_partial_mapping=None, bowtie2_mode=None,
              fusion_search=None, fusion_anchor_length=None, fusion_min_dist=None, fusion_read_mismatches=None,
              fusion_multireads=None, fusion_multipairs=None, fusion_ignore_chromosomes=None, rg_id=None,
              rg_sample=None, rg_library=None, rg_description=None, rg_platform_unit=None, rg_center=None,
              rg_date=None, rg_platform=None):
    
        """
        Usage:
        tophat [options] <bowtie_index> <reads1[,reads2,...]> [reads1[,reads2,...]] \
                                        [quals1,[quals2,...]] [quals1[,quals2,...]]

        Options:
            -v/--version
            -o/--output-dir                <string>    [ default: ./tophat_out         ]
            --bowtie1                                  [ default: bowtie2              ]
            -N/--read-mismatches           <int>       [ default: 2                    ]
            --read-gap-length              <int>       [ default: 2                    ]
            --read-edit-dist               <int>       [ default: 2                    ]
            --read-realign-edit-dist       <int>       [ default: "read-edit-dist" + 1 ]
            -a/--min-anchor                <int>       [ default: 8                    ]
            -m/--splice-mismatches         <0-2>       [ default: 0                    ]
            -i/--min-intron-length         <int>       [ default: 50                   ]
            -I/--max-intron-length         <int>       [ default: 500000               ]
            -g/--max-multihits             <int>       [ default: 20                   ]
            --suppress-hits
            -x/--transcriptome-max-hits    <int>       [ default: 60                   ]
            -M/--prefilter-multihits                   ( for -G/--GTF option, enable
                                                         an initial bowtie search
                                                         against the genome )
            --max-insertion-length         <int>       [ default: 3                    ]
            --max-deletion-length          <int>       [ default: 3                    ]
            --solexa-quals
            --solexa1.3-quals                          (same as phred64-quals)
            --phred64-quals                            (same as solexa1.3-quals)
            -Q/--quals
            --integer-quals
            -C/--color                                 (Solid - color space)
            --color-out
            --library-type                 <string>    (fr-unstranded, fr-firststrand,
                                                        fr-secondstrand)
            -p/--num-threads               <int>       [ default: 1                   ]
            -R/--resume                    <out_dir>   ( try to resume execution )
            -G/--GTF                       <filename>  (GTF/GFF with known transcripts)
            --transcriptome-index          <bwtidx>    (transcriptome bowtie index)
            -T/--transcriptome-only                    (map only to the transcriptome)
            -j/--raw-juncs                 <filename>
            --insertions                   <filename>
            --deletions                    <filename>
            -r/--mate-inner-dist           <int>       [ default: 50                  ]
            --mate-std-dev                 <int>       [ default: 20                  ]
            --no-novel-juncs
            --no-novel-indels
            --no-gtf-juncs
            --no-coverage-search
            --coverage-search
            --microexon-search
            --keep-tmp
            --tmp-dir                      <dirname>   [ default: <output_dir>/tmp ]
            -z/--zpacker                   <program>   [ default: gzip             ]
            -X/--unmapped-fifo                         [use mkfifo to compress more temporary
                                                         files for color space reads]

        Advanced Options:
            --report-secondary-alignments
            --no-discordant
            --no-mixed

            --segment-mismatches           <int>       [ default: 2                ]
            --segment-length               <int>       [ default: 25               ]

            --bowtie-n                                 [ default: bowtie -v        ]
            --min-coverage-intron          <int>       [ default: 50               ]
            --max-coverage-intron          <int>       [ default: 20000            ]
            --min-segment-intron           <int>       [ default: 50               ]
            --max-segment-intron           <int>       [ default: 500000           ]
            --no-sort-bam                              (Output BAM is not coordinate-sorted)
            --no-convert-bam                           (Do not output bam format.
                                                        Output is <output_dir>/accepted_hit.sam)
            --keep-fasta-order
            --allow-partial-mapping

        Bowtie2 related options:
          Preset options in --end-to-end mode (local alignment is not used in TopHat2)
            --b2-very-fast
            --b2-fast
            --b2-sensitive
            --b2-very-sensitive

          Alignment options
            --b2-N                         <int>       [ default: 0                ]
            --b2-L                         <int>       [ default: 20               ]
            --b2-i                         <func>      [ default: S,1,1.25         ]
            --b2-n-ceil                    <func>      [ default: L,0,0.15         ]
            --b2-gbar                      <int>       [ default: 4                ]

          Scoring options
            --b2-mp                        <int>,<int> [ default: 6,2              ]
            --b2-np                        <int>       [ default: 1                ]
            --b2-rdg                       <int>,<int> [ default: 5,3              ]
            --b2-rfg                       <int>,<int> [ default: 5,3              ]
            --b2-score-min                 <func>      [ default: L,-0.6,-0.6      ]

          Effort options
            --b2-D                         <int>       [ default: 15               ]
            --b2-R                         <int>       [ default: 2                ]

        Fusion related options:
            --fusion-search
            --fusion-anchor-length         <int>       [ default: 20               ]
            --fusion-min-dist              <int>       [ default: 10000000         ]
            --fusion-read-mismatches       <int>       [ default: 2                ]
            --fusion-multireads            <int>       [ default: 2                ]
            --fusion-multipairs            <int>       [ default: 2                ]
            --fusion-ignore-chromosomes    <list>      [ e.g, <chrM,chrX>          ]

            --fusion-do-not-resolve-conflicts          [this is for test purposes  ]

        SAM Header Options (for embedding sequencing run metadata in output):
            --rg-id                        <string>    (read group ID)
            --rg-sample                    <string>    (sample ID)
            --rg-library                   <string>    (library ID)
            --rg-description               <string>    (descriptive string, no tabs allowed)
            --rg-platform-unit             <string>    (e.g Illumina lane ID)
            --rg-center                    <string>    (sequencing center name)
            --rg-date                      <string>    (ISO 8601 date of the sequencing run)
            --rg-platform                  <string>    (Sequencing platform descriptor)

            for detailed help see http://tophat.cbcb.umd.edu/manual.html
        """
        options = ""
        options += " -o %s" % out_dir
        options += " --read-mismatches %i" % read_mismatches if read_mismatches else ""
        options += " --read-gap-length %i" % read_gap_length if read_gap_length else ""
        options += " --read-edit-dist %i" % read_edit_dist if read_edit_dist else ""
        options += " --read-realign-edit-dist %i" % read_realign_edit_dist if read_realign_edit_dist else ""
        options += " --min-anchor %i" % min_anchor if min_anchor else ""
        options += " --splice-mismatches %i" % splice_mismatches if splice_mismatches else ""
        options += " --min-intron-length %i" % min_intron_length if min_intron_length else ""
        options += " --max-intron-length %i" % max_intron_length if max_intron_length else ""
        options += " --max-multihits %i" % max_multihits if max_multihits else ""

        options += " --transcriptome-max-hits %i" % transcriptome_max_hits if transcriptome_max_hits else ""
        options += " --max-insertion-length %i" % max_insertion_length if max_insertion_length else ""
        options += " --max-deletion-length %i" % max_deletion_length if max_deletion_length else ""

        options += " --phred64" if quality == "phred64" else ""
        options += " --num-threads %i" % self.threads
        options += " --library-type %s" % library_type if library_type else ""
        options += " --GTF %s" % known_transcripts_file if known_transcripts_file else ""
        options += " --transcriptome-index %s" % transcriptome_index if transcriptome_index else ""
        options += " --raw-juncs %s" % raw_juncs_file if raw_juncs_file else ""
        options += " --insertions %s" % insertions_file if insertions_file else ""
        options += " --deletions %s" % deletions_file if deletions_file else ""
        options += " --mate-inner-dist %i" % mate_inner_dist if mate_inner_dist else ""
        options += " --mate-std-dev %i" % mate_std_dev if mate_std_dev else ""
        options += " --no-novel-juncs" if no_novel_juncs else ""
        options += " --no-novel-indels" if no_novel_indels else ""
        options += " --no-gtf-juncs" if no_gtf_juncs else ""
        options += " --no-coverage-search" if no_coverage_search else ""
        options += " --coverage-search" if coverage_search else ""
        options += " --microexon-search" if microexon_search else ""
        options += " --keep-tmp" if keep_tmp else ""
        options += " --tmp-dir %s" % tmp_dir if tmp_dir else ""

        options += " --report-secondary-alignments" if report_secondary_alignments else ""
        options += " --no-discordant" if no_discordant else ""
        options += " --no-mixed" if no_mixed else ""

        options += " --segment-mismatches %i" % segment_mismatches if segment_mismatches else ""
        options += " --segment-length %i" % segment_length if segment_length else ""
        options += " --bowtie-n" if bowtie_n else ""
        options += " --min-coverage-intron %i" % min_coverage_intron if min_coverage_intron else ""
        options += " --max-coverage-intron %i" % max_coverage_intron if max_coverage_intron else ""
        options += " --min-segment-intron %i" % min_segment_intron if min_segment_intron else ""
        options += " --max-segment-intron %i" % max_segment_intron if max_segment_intron else ""
        options += " --no-sort-bam" if no_sort_bam else ""
        options += " --no-convert-bam" if no_convert_bam else ""
        options += " --keep-fasta-order" if keep_fasta_order else ""
        options += " --allow-partial-mapping" if allow_partial_mapping else ""

        options += " --b2-%s" % bowtie2_mode if bowtie2_mode else ""
        #allowed mode: very-fast, fast, sensitive, very-sensitive

        options += " --fusion-search" if fusion_search else ""
        options += " --fusion-anchor-length %i" % fusion_anchor_length if fusion_anchor_length else ""
        options += " --fusion-min-dist %i" % fusion_min_dist if fusion_min_dist else ""
        options += " --fusion-read-mismatches %i" % fusion_read_mismatches if fusion_read_mismatches else ""
        options += " --fusion-multireads %i" % fusion_multireads if fusion_multireads else ""
        options += " --fusion-multipairs %i" % fusion_multipairs if fusion_multipairs else ""
        options += " --fusion-ignore-chromosomes %s" % fusion_ignore_chromosomes if fusion_ignore_chromosomes else ""

        options += " --rg-id %s" % rg_id if rg_id else ""
        options += " --rg-sample %s" % rg_sample if rg_sample else ""
        options += " --rg-library %s" % rg_library if rg_library else ""
        options += " --rg-description %s" % rg_description if rg_description else ""
        options += " --rg-platform-unit %s" % rg_platform_unit if rg_platform_unit else ""
        options += " --rg-center %s" % rg_center if rg_center else ""
        options += " --rg-date %s" % rg_date if rg_date else ""
        options += " --rg-platform %s" % rg_platform if rg_platform else ""

        options += " %s" % index
        options += " %s" % left_reads
        options += " %s" % right_reads if right_reads else ""

        self.execute(options, cmd="tophat")


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from collections import Iterable
from subprocess import PIPE, Popen
from RouToolPa.Tools.Abstract import Tool



class Valgrind(Tool):
    """
    Memory usage tester and profiler
    """
    def __init__(self, path="", max_threads=4):

        Tool.__init__(self, "valgrind", path=path, max_threads=max_threads)

    @staticmethod
    def parse_options(cmd, cmd_log=None, massif_mode=False,  count_all_memory=False,
                      valgrind_log_file=None, massif_report_file=None):
        options = ""
        options += " --tool=massif" if massif_mode else ""
        options += " --pages-as-heap=yes" if count_all_memory else ""
        options += " --log-file=%s" % valgrind_log_file if valgrind_log_file else ""
        options += " --massif-out-file=%s" % massif_report_file if massif_report_file else ""
        options += " -b %s" % cmd
        options += " > %s 2>&1" % cmd_log if cmd_log else ""

        return options

    def ms_print(self, massif_report, ms_print_report):

        options = " %s" % massif_report
        options += " > %s" % ms_print_report

        self.execute(options=options, cmd="ms_print")

    def measure_ram_usage(self, cmd, output_prefix, count_all_memory=True):

        cmd_log = "%s.cmd.log" % output_prefix
        valgrind_log = "%s.valgrind.log" % output_prefix
        massif_report = "%s.valgrind.massif.out" % output_prefix
        ms_print_report = "%s.valgrind.massif.ms_print.out" % output_prefix

        options = self.parse_options(cmd, cmd_log=cmd_log, massif_mode=True,  count_all_memory=count_all_memory,
                                     valgrind_log_file=valgrind_log, massif_report_file=massif_report)

        self.execute(options=options)
        self.ms_print(massif_report, ms_print_report)

if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Spades(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "spades.py", path=path, max_threads=max_threads)

    def assembly(self, pacbio_reads=None, sanger_reads=None, only_error_correction=False,
                 only_assembler=False, careful=True, single_cell=False, iontorrent=False,
                 trusted_contigs=None, untrusted_contigs=None, out_dir="spades"):
        options = " -t %i" % self.threads
        options += " --sanger %s" % (sanger_reads) if sanger_reads else ""
        options += " --pacbio %s" % (pacbio_reads) if pacbio_reads else ""
        options += " --only-error-correction" if only_error_correction else ""
        options += " --only_assembler" if only_assembler else ""
        options += " --careful" if careful else ""
        options += " --sc" if single_cell else ""
        options += " --iontorrent" if iontorrent else ""
        options += " -o %s" % out_dir
        options += " --trusted-contigs %s" % trusted_contigs if trusted_contigs else ""
        options += " --untrusted-contigs %s" % untrusted_contigs if untrusted_contigs else ""
        options += " "
        options += " "
        options += " "
        self.execute(options)

    def error_correction(self):
        pass

if __name__ == "__main__":
    pass


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import Tool


class SSPACE(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "SSPACE_Standard_v3.0.pl", path=path, max_threads=max_threads)

    def scaffold(self, library_file, contig_fasta, output_basename, extend_contigs=False, min_overlap_len=None,
                 min_reads_for_contig_extension=None, min_contig_len=None, min_link_number=None,
                 min_link_ratio=None, min_contig_overlap=None, max_gaps_for_bowtie=None, skip_read_processing=None,
                 verbose_scaffolding=None,
                 make_dot_file=True):
        """
        IMPORTANT: Extension of contigs during scaffolding doesnt work in SSPACE 3.0!!!!!!!!!!!!!

        """

        options = " -T %i" % self.threads
        options += " -l %s" % library_file
        options += " -s %s" % contig_fasta
        options += " -x 1" if extend_contigs else ""
        options += " -m %i" % min_overlap_len if min_overlap_len else ""
        options += " -o %i" % min_reads_for_contig_extension if min_reads_for_contig_extension else ""
        options += " -z %i" % min_contig_len if min_contig_len else ""
        options += " -k %i" % min_link_number if min_link_number else ""
        options += " -a %f" % min_link_ratio if min_link_ratio else ""
        options += " -n %i" % min_contig_overlap if min_contig_overlap else ""
        options += " -g %i" % max_gaps_for_bowtie if max_gaps_for_bowtie else ""
        options += " -S" if skip_read_processing else ""
        options += " -b %s" % output_basename if output_basename else ""
        options += " -v 1" if verbose_scaffolding else ""
        options += " -p 1" if make_dot_file else ""

        self.execute(options)

if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from collections import Iterable, OrderedDict
from math import sqrt

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np

from RouToolPa.Parsers.Abstract import Header, Collection


class HaploviewHeader(list, Header):

    def __str__(self):
        """
        :return: vcf-like string representation of header
        """
        return "#" + "\t".join(self)


class HaploviewCollection(Collection):

    def __init__(self, metadata=None, records_dict=None, header=None, input_file=None, from_file=False):
        # metadata should be Metadata class
        # header should be Header class
        # record_list should be list of Record class
        if from_file:
            self.read(input_file)
        else:
            self.records = OrderedDict() if records_dict is None else records_dict
            self.metadata = metadata
            self.header = header
        self.scaffold_list = self.scaffolds()
        self.scaffold_length = self.scaffold_len()
        self.number_of_scaffolds = len(self.scaffold_list)
        self.record_index = self.rec_index()

    def __str__(self):
        pass

    def __len__(self):
        return self.record_index[self.scaffold_list[-1]][1] + 1

    def read(self, input_file, parsing_mode="full"):
        """
        :param input_file:
        :param parsing_mode: full - parse all fields, short - parse coordinates, D' and r2
        :return:
        """
        with open(input_file, "r") as in_fd:
            if parsing_mode == "full":
                self.header = HaploviewHeader(in_fd.readline().strip().split("\t"))
            else:
                self.header = HaploviewHeader(["L1", "L2", "D'", "r^2"])
            current_chr = None
            for line in in_fd:
                tmp = line.strip().split()
                chr, first_snp = tmp[0].split(":")
                if (current_chr is None) or (chr != current_chr):
                    self.records[chr] = []
                    if current_chr is not None:
                        self.records[current_chr] = np.array(self.records[current_chr])
                    current_chr = chr
                second_snp = tmp[1].split(":")[1]
                if parsing_mode == "full":
                    tmp = [int(first_snp), int(second_snp)] + map(float, tmp[2:])
                    self.parsing_mode = "full"
                elif parsing_mode == "short":
                    tmp = [int(first_snp), int(second_snp), float(tmp[2]), float(tmp[4])]
                    self.parsing_mode = "short"
                self.records[chr].append(tmp)

    def filter(self, expression):
        filtered_dict = OrderedDict()
        filtered_out_dict = OrderedDict()

        for scaffold in self.records:
            filtered_dict[scaffold] = []
            for record in self.records[scaffold]:
                if expression(record):
                    filtered_dict[scaffold].append(record)
                else:
                    filtered_out_dict[scaffold].append(record)

            if filtered_dict[scaffold]:
                filtered_dict[scaffold] = np.array(filtered_dict[scaffold])
            else:
                filtered_dict.pop(scaffold, None)

            if filtered_out_dict[scaffold]:
                filtered_out_dict[scaffold] = np.array(filtered_out_dict[scaffold])
            else:
                filtered_out_dict.pop(scaffold, None)

        return HaploviewCollection(records_dict=filtered_dict), HaploviewCollection(records_dict=filtered_out_dict)

    @staticmethod
    def filter_record_by_snp_distance(record, max_dist):
        if (record[1] - record[0]) <= max_dist:
            return True
        return False

    def filter_by_snp_distance(self, max_dist):

        def expression(record):
            self.filter_record_by_snp_distance(record, max_dist)

        return self.filter(expression)

    def write_as_tsv(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#Scaffold\t" + "\t".join(self.header) + "\n")
            for scaffold in self.records:
                for record in self.records[scaffold]:
                    out_fd.write("%s\t%s\n" % (scaffold, "\t".join(map(str, record))))

"""
    def filter_records(self, expression):
        # expression should be a function with one argument - record entry
        filtered_records = OrderedDict({})
        filtered_out_records = OrderedDict({})
        for scaffold in self.scaffold_list:
            filtered_records[scaffold] = []
            filtered_out_records[scaffold] = []
            for record in self.records[scaffold]:
                if expression(record):
                    filtered_records[scaffold].append(record)
                else:
                    filtered_out_records[scaffold].append(record)
        return filtered_records, filtered_out_records

    def write(self, output_file, desired_scaffolds=None):
        with open(output_file, "w") as out_fd:
            if self.metadata:
                out_fd.write(str(self.metadata) + "\n")
            if self.header:
                out_fd.write(str(self.header) + "\n")
            scaffolds = desired_scaffolds if desired_scaffolds else self.scaffold_list
            for scaffold in scaffolds:
                for record in self.records[scaffold]:
                    out_fd.write(scaffold + "\t" + str(record) + "\n")

    def get_location(self, annotation_dict, key="Loc", use_synonym=False, strand_key="strand",
                     synonym_dict=None, feature_type_black_list=[]):
        for scaffold in self.records:
            for record in self.records[scaffold]:
                record.get_location(scaffold, annotation_dict, key=key, use_synonym=use_synonym,
                                    synonym_dict=synonym_dict, feature_type_black_list=feature_type_black_list,
                                    strand_key=strand_key)

    def set_location_flag(self, annotation_dict, expression, flag):
        for scaffold in self.records:
            for record in self.records[scaffold]:
                record.set_location_flag(scaffold, annotation_dict, expression, flag)

    def split_records_by_flags(self, flag_set, mode="all"):
        # possible modes:
        # all - record to be counted as 'with flag' must have all flags from flags_list
        # one - record to be counted as 'with flag' must have at least one flag from flags_list
        flags = set(flag_set)
        return self.filter_records(lambda record: flags & record.flags == flags if mode == "all" else flags & record.flags)

    def split_by_flags(self, flag_set, mode="all"):
        # possible modes:
        # all - record to be counted as 'with flag' must have all flags from flags_list
        # one - record to be counted as 'with flag' must have at least one flag from flags_list
        flags = set(flag_set)
        return self.filter(lambda record: flags & record.flags == flags if mode == "all" else flags & record.flags)

    def count_locations(self, annotation_black_list=[],
                        allow_several_counts_of_record=False,
                        out_filename="location_counts.t",
                        write=True,
                        count_dir="location_counts"):
        os.system("mkdir -p %s" % count_dir)
        region_counts_dict = TwoLvlDict({})
        for region in self.records:
            count_locations_dict = {"igc": 0, "unknown": 0}

            for record in self.records[region]:
                if "Loc" not in record.info_dict:
                    count_locations_dict["unknown"] += 1
                    continue
                if not record.info_dict["Loc"]:
                    count_locations_dict["unknown"] += 1
                    continue
                if allow_several_counts_of_record:
                    for location in record.info_dict["Loc"]:
                        if location in annotation_black_list:
                            continue
                        if location not in count_locations_dict:
                            count_locations_dict[location] = 1
                        else:
                            count_locations_dict[location] += 1
                else:
                    full_location = []
                    for location in record.info_dict["Loc"]:
                        if location in annotation_black_list:
                            continue
                        full_location.append(location)
                    if not full_location:
                        continue
                    full_location.sort()
                    full_location = "/".join(full_location)
                    if full_location not in count_locations_dict:
                        count_locations_dict[full_location] = 1
                    else:
                        count_locations_dict[full_location] += 1

            labels = []
            counts = []
            for location in count_locations_dict:
                if count_locations_dict[location] == 0 or location in annotation_black_list:
                    continue
                labels.append(location)
                counts.append(count_locations_dict[location])
            region_counts_dict[region] = OrderedDict([(label, count) for label, count in zip(labels, counts)])

        if write:
            region_counts_dict.write("%s/%s" % (count_dir, out_filename))
        return region_counts_dict

    def location_pie(self, pie_name="Location of variants", annotation_colors=[],
                     dpi=150, figsize=(30, 30),
                     explode=True, annotation_black_list=[],
                     allow_several_counts_of_record=False,
                     pie_prefix="variant_location_pie1",
                     full_genome_pie_prefix="variant_location_pie_full_genome",
                     counts_filename="location_counts.t",
                     plot_dir="variant_location_pie",
                     counts_dir="location_counts",
                     radius = 1,
                     draw_percents_on_single_pie=False,
                     combine_mixed=False,
                     extension_list=["svg", "eps", "pdf", "png", "jpg"]):
        print("Drawing location pie...")

        os.system("mkdir -p %s" % plot_dir)
        reference_colors = {"CDS": "#FBFD2B",    # yellow
                            "5'_UTR": "#FF000F",
                            "five_prime_UTR": "#FF000F",     # red
                            "3'_UTR": "#000FFF",
                            "three_prime_UTR": "#000FFF",     # blue
                            "igc": "#4ED53F",     # green
                            "ncRNA": 'cyan',
                            "other": "#ADB696",
                            "tRNA": "magenta"
                            }

        if annotation_colors:
            reference_colors = annotation_colors
            reference_colors["other"] = "#ADB696"

        count_locations_dict = self.count_locations(annotation_black_list=annotation_black_list,
                                                    allow_several_counts_of_record=allow_several_counts_of_record,
                                                    out_filename=counts_filename,
                                                    write=True,
                                                    count_dir=counts_dir)
        num_of_regions = len(count_locations_dict)
        side = int(sqrt(num_of_regions))
        if side*side != num_of_regions:
            side += 1
        sub_plot_dict = OrderedDict({})
        fig = plt.figure(2, dpi=dpi, figsize=figsize)
        fig.suptitle(pie_name, fontsize=20)

        index = 1
        all_labels = []
        all_counts = []
        all_colors = []

        for region in count_locations_dict:
            sub_plot_dict[region] = plt.subplot(side, side, index, axisbg="#D6D6D6")
            labels = []
            counts = []
            colors = []
            if combine_mixed:
                labels.append("other")
                counts.append(0)
                colors.append(reference_colors["other"])
            for label in count_locations_dict[region]:
                if count_locations_dict[region][label] == 0 or label in annotation_black_list:
                    continue
                if combine_mixed and "/" in label:
                    counts[0] += count_locations_dict[region][label]
                    continue

                labels.append(label)
                counts.append(count_locations_dict[region][label])
                if label not in reference_colors:
                    colors.append(reference_colors["other"])
                else:
                    colors.append(reference_colors[label])

            explodes = np.zeros(len(counts))
            if explode and counts:
                max_count_index = counts.index(max(counts))
                explodes[max_count_index] = 0.1
            plt.pie(counts, explode=explodes, labels=labels, colors=colors,
                    autopct='%1.1f%%', shadow=True, startangle=90)
            plt.title("Region %s" % region, y=1.08)
            # Set aspect ratio to be equal so that pie is drawn as a circle.
            plt.axis('equal')
            index += 1
            for i in range(0, len(labels)):
                if labels[i] not in all_labels:
                    all_labels.append(labels[i])
                    all_counts.append(counts[i])
                    all_colors.append(colors[i])
                else:
                    label_index = all_labels.index(labels[i])
                    all_counts[label_index] += counts[i]

        for extension in extension_list:
            plt.savefig("%s/%s.%s" % (plot_dir, pie_prefix, extension))
        plt.close()

        fig = plt.figure(3, dpi=dpi, figsize=(4, 4))
        fig.suptitle(pie_name, fontsize=20)
        plt.subplot(1, 1, 1, axisbg="#D6D6D6")
        all_explodes = np.zeros(len(all_counts))
        if explode and all_counts:
            max_count_index = all_counts.index(max(all_counts))
            all_explodes[max_count_index] = 0.1
        if len(all_labels) > 0:
            max_label_length = max([len(x) for x in all_labels])
            max_count = max(all_counts)
            max_letters = 1
            while int(max_count / 10**max_letters) != 0:
                max_letters += 1
        if draw_percents_on_single_pie:
            patches, texts, autotexts = plt.pie(all_counts, explode=all_explodes, colors=all_colors,
                                                autopct='%1.1f%%', shadow=True, startangle=90, radius=2)
        else:
            patches, texts = plt.pie(all_counts, explode=all_explodes, colors=all_colors,
                                     shadow=True, startangle=90, radius=2)
        percent = 100 * np.array(all_counts).astype(np.float32, copy=False)/sum(all_counts)

        labels = ['{0}  -  {1} ({2:1.2f}%)'.format(i.ljust(max_label_length), str(j).ljust(max_letters), k) for i, j, k in zip(all_labels, all_counts, percent)]

        if len(all_labels) > 0:
            sort_legend = True
            if sort_legend:
                patches, labels, dummy = zip(*sorted(zip(patches, labels, all_counts),
                                                     key=lambda x: x[2],
                                                     reverse=True))

        plt.legend(patches, labels, loc='center left',  fontsize=13, bbox_to_anchor=(-1.0, 0.5))
        # plt.title("Full genome")
        plt.axis('equal')
        for extension in extension_list:
            plt.savefig("%s/%s.%s" % (plot_dir, full_genome_pie_prefix, extension), bbox_inches='tight')
        plt.close()
"""


#!/usr/bin/env python

from RouToolPa.Tools.Abstract import JavaTool
from RouToolPa.Collections.General import SynDict


class SNPeff(JavaTool):
    def __init__(self, max_threads=4, jar_path="", jar="snpEff.jar",
                 max_memory="500m", config_path=""):
        JavaTool.__init__(self, jar, java_path="", max_threads=max_threads, jar_path=jar_path, max_memory=max_memory)
        self.config_path = config_path

    def build_db(self, datafiles, input_format="gff3", build_type=None):
        """
        To build custom db at first add record to config file

        # Mouse genome, version mm37.61
        mm37.61.genome : Mouse

        Build DB options:
            input_format:
            -embl                   : Use Embl format. It implies '-1'.
            -genbank                : Use GenBank format. It implies '-1'.
            -gff2                   : Use GFF2 format (obsolete). It implies '-1'.
            -gff3                   : Use GFF3 format. It implies '-1'
            -gtf22                  : Use GTF 2.2 format. It implies '-1'. Default
            -refseq                 : Use RefSeq table from UCSC. It implies '-0'.
            -txt                    : Use TXT format (obsolete).

            build_type
            -knowngenes             : Use KnownGenes table from UCSC. It implies '-0'.
            -onlyReg                : Only build regulation tracks.
            -cellType <type>        : Only build regulation tracks for cellType <type>.

        Generic options:
            -0                      : File positions are zero-based (same as '-inOffset 0 -outOffset 0')
            -1                      : File positions are one-based (same as '-inOffset 1 -outOffset 1')
            -c , -config            : Specify config file
            -h , -help              : Show this help and exit
            -if, -inOffset          : Offset input by a number of bases. E.g. '-inOffset 1' for one-based input files
            -of, -outOffset         : Offset output by a number of bases. E.g. '-outOffset 1' for one-based output files
            -noLog                  : Do not report usage statistics to server
            -q , -quiet             : Quiet mode (do not show any messages or errors)
            -v , -verbose           : Verbose mode
        """

        options = "-v -%s" % input_format
        options += " %s" % build_type if build_type else ""
        options += " " + " ".join(datafiles) if isinstance(datafiles, list) else " " + datafiles

        self.execute(options, cmd="build")

    def annotate(self, genome, input_vcf, output_vcf, summary_file=None, verbose=True, no_downstream=False,
                 no_intergenic=False, no_intron=False, no_upstream=False, no_utr=False):

        options = " -v" if verbose else ""
        options += " -s %s" % summary_file if summary_file else ""
        options += " -no-downstream" if no_downstream else ""
        options += " -no-intergenic" if no_intergenic else ""
        options += " -no-intron" if no_intron else ""
        options += " -no-upstream" if no_upstream else ""
        options += " -no-utr" if no_utr else ""
        options += " %s" % genome
        options += " %s" % input_vcf
        options += " > %s" % output_vcf

        self.execute(options, cmd="")

    def add_gene_synonyms(self, input_file, output_file, synonym_file, key_column=0, value_column=1,
                          header_name_for_synonym="Common_name", snpeff_tab_column_id_column=8):
        synonym_dict = SynDict(filename=synonym_file,
                               key_index=key_column,
                               value_index=value_column,
                               comments_prefix="#")
        #print synonym_dic
        with open(input_file, "r") as in_fd, open(output_file, "w") as out_fd:
            header = in_fd.readline().strip() + "\t%s\n" % header_name_for_synonym
            out_fd.write(header)

            for line in in_fd:
                tmp = line.strip().split("\t")
                #print tmp
                gene_name = tmp[snpeff_tab_column_id_column]
                #print gene_name
                #print gene_name
                #print synonym_dict[gene_name]
                #print gene_name
                #if gene_name in synonym_dict:
                #    print "AAAAAAAA"
                tmp.append(synonym_dict[gene_name] if gene_name in synonym_dict else "")
                out_fd.write("\t".join(tmp) + "\n")



if __name__ == "__main__":
    import os
    workdir = "/home/mahajrod/genetics/desaminases/data/S288C_R64/SNPeff"
    SNPeff_path = "/home/mahajrod/Repositories/genetic/NGS_tools/snpEff"
    SNPeff = SNPeff(jar_path=SNPeff_path)

    #print(SNPeff.jar_path)
    #os.chdir(workdir)
    #gff_file = "/home/mahajrod/genetics/desaminases/data/S288C_R64/merged_saccharomyces_cerevisiae_R64-1-1_20110208_Nagalakshmi_UTR_2008.gff3"
    #SNPeff.build_db(gff_file, input_format="gff3")

    """
    workdir = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/intersect_S288C/normal_regions"
    ref_genome = "S288C_R64_UTR"
    in_file = "in_normal_regions_homo.vcf"
    out_file_suffix = "_S288C_R64_UTR_annotated.vcf"
    """

    workdir = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/intersect_LAN210_v0.10m/normal_regions"
    ref_genome = "LAN210_v0.10m"
    in_file = "variants_normal_regions_homo.vcf"
    out_file_suffix = "_LAN210_v0.10m_annotated.vcf"

    os.chdir(workdir)

    #os.system("mkdir -p %s" % ref_genome)
    #os.chdir(ref_genome)

    SNPeff.annotate(ref_genome, "../" + in_file, in_file[:-4] + out_file_suffix)


__author__ = 'mahajrod'

from Bio import AlignIO
from Bio.Phylo.Consensus import bootstrap
from RouToolPa.Routines.MultipleAlignment import MultipleAlignmentRoutines


class PhylogeneticsRoutines(MultipleAlignmentRoutines):
    def __init__(self):
        MultipleAlignmentRoutines.__init__(self)

    def bootstrap_alignment(self, alignment_file, output_directory, output_prefix,
                            replicate_number, format="fasta"):
        self.safe_mkdir(output_directory)
        alignment = AlignIO.read(alignment_file, format=format)

        bootstrap_replicate_generator = bootstrap(alignment, replicate_number)

        i = 0
        for replicate in bootstrap_replicate_generator:
            i += 1
            output_file = "%s/%s_%i.%s" % (output_directory, output_prefix, i, format)
            AlignIO.write(replicate, output_file, format=format)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Platanus(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "platanus", path=path, max_threads=max_threads)

    def assemble_unitigs(self, reads_files, out_prefix="out", memory_limit=None,
                         kmer_extension_step=None, branch_cutting_diference=None,
                         bubble_crush_difference=None, initial_kmer_coverage_cutoff=None,
                         min_kmer_coverage=None, kmer_extension_safety_level=None,
                         unitig_assembly_log="unitig_assembly.log"):
        # TODO: add -k and -K options

        options = " -t %i" % self.threads
        options += " -o %s" % out_prefix
        options += " -m %i" % memory_limit if memory_limit else ""
        options += " -d %f" % branch_cutting_diference if branch_cutting_diference else ""
        options += " -u %f" % bubble_crush_difference if bubble_crush_difference else ""
        options += " -s %i" % kmer_extension_step if kmer_extension_step else ""
        options += " -n %i" % initial_kmer_coverage_cutoff if initial_kmer_coverage_cutoff else ""
        options += " -c %i" % min_kmer_coverage if min_kmer_coverage else ""
        options += " -a %f" % kmer_extension_safety_level if kmer_extension_safety_level else ""
        options += " "
        options += " "
        reads_string = ""
        for reads in reads_files:
            if isinstance(reads, str):
                reads_string += " %s" % reads
            else:
                reads_string += " " + " ".join(reads)
        options += " -f %s" % reads_string
        options += " > %s" % unitig_assembly_log
        self.execute(options, cmd="platanus assemble")

    def scaffold(self, contig_file, bubble_file, pe_reads_list=None, mp_reads_list=None,
                 out_prefix="out", min_insert_size=None,scaffolding_log="scaffolding.log"):

        if (not pe_reads_list) and (not mp_reads_list):
            raise ValueError("Both PE and MP read files were not set")

        options = " -t %i" % self.threads
        options += " -o %s" % out_prefix
        options += " -c %s" % contig_file
        options += " -b %s" % bubble_file
        options += " -n %i" % min_insert_size
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " > %s" % scaffolding_log

        """
         Map paired-end (mate-pair) reads on contigs, determine the order of contigs
    and construct scaffolds.


    INPUT OPTIONS

    -ip{INT} PAIR1 [PAIR2 ...]         : Lib_id inward_pair_file (reads in 1 file, fasta or fastq)
                                         Ex. -ip1 lib1.fa

    -IP{INT} FWD1 REV1 [FWD2 REV2 ...] : Lib_id inward_pair_files (reads in 2 files, fasta or fastq)
                                         Ex. -IP1 lib1_1.fa lib1_2.fa

    -op{INT} PAIR1 [PAIR2 ...]         : Lib_id outward_pair_file (reads in 1 file, fasta or fastq)
                                         Ex. -op1 lib1.fa

    -OP{INT} FWD1 REV1 [FWD2 REV2 ...] : lib_id outward_pair_files (reads in 2 files, fasta or fastq)
                                         Ex. -OP1 lib1_1.fa lib1_2.fa

                                         The file format is automatically detected.
                                         see "***** NOTE ***** Paired-end (Mate-pair) input" below


    OTHER OPTIONS

    -a{INT1} INT2 : lib_id average_insert_size
                    Fixed average insert size (INT) is used instead of auto estimation.

    -d{INT1} INT2 : lib_id SD_insert_size
                    Fixed SD of insert size (INT) is used instead of auto estimation.

    -s INT        : Mapping seed length (default 32)
                    Seed length must not be larger than reads length. Smaller INT
                    decrease speed.

    -v INT        : Minimum overlap length (default 32)
                    If adjacent contigs have overlap (length >= INT) and properly
                    close to each other, the contigs are joined.

    -l INT        : Minimum number of link (default 3)
                    Platanus first estimates the threshold of link (number) and
                    makes scaffolds, then decreases the threshold to INT and
                    extends scaffolds.

    -u FLOAT      : Maximum difference for bubble crush (identity, default 0.1)
                    Larger FLOAT increases the number of bubbles merged. If
                    heterozygosity of the sample is high, large FLOAT may be
                    suitable (Ex. -u 0.2).


    OUTPUT FILES

    PREFIX_scaffold.fa           : assembled sequences that include gaps('N's mean gaps)

    PREFIX_scaffoldBubble.fa     : removed bubble sequences

    PREFIX_scaffoldComponent.tsv : the information about composition of scaffolds
                                   (i.e. which contigs constitute a scaffold)

--------------------------------------------------------------------------------


        """


        self.execute(options, cmd="platanus scaffold")

    def gap_close(self, tool_options):
        options = ""
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        self.execute(options, cmd="custom command if necessary ex: tool_command subcommand")
if __name__ == "__main__":
    pass


__author__ = 'mahajrod'

from RouToolPa.Tools.Filter.FaCut import FaCut
from RouToolPa.Tools.Filter.FastQC import FastQC
from RouToolPa.Tools.Filter.Adapters import adapters_PE
from RouToolPa.Tools.Filter.Cutadapt import Cutadapt
from RouToolPa.Tools.Filter.Clumpify import Clumpify
from RouToolPa.Tools.Filter.TrimGalore import TrimGalore
from RouToolPa.Tools.Filter.Trimmomatic import Trimmomatic
from RouToolPa.Tools.Filter.Cookiecutter import Cookiecutter
from RouToolPa.Tools.Filter.Cookiecutter import CookiecutterOld


FaCut = FaCut()
FastQC = FastQC()
Cutadapt = Cutadapt()
Clumpify = Clumpify()
TrimGalore = TrimGalore()
Trimmomatic = Trimmomatic()
Cookiecutter = Cookiecutter()
CookiecutterOld = CookiecutterOld()




#!/usr/bin/env python

__author__ = 'mahajrod'

import os
from RouToolPa.Tools.Abstract import JavaTool
from RouToolPa.Routines import VCFRoutines, FileRoutines


class HaplotypeCaller(JavaTool):
    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T HaplotypeCaller", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    @staticmethod
    def parse_options_for_parallel_run(reference, alignment, genotyping_mode="DISCOVERY", output_mode="EMIT_VARIANTS_ONLY",
                                       stand_call_conf=30, gvcf_mode=False):

        options = " -R %s" % reference
        options += " -I %s" % alignment
        options += " --genotyping_mode %s" % genotyping_mode if genotyping_mode else ""
        options += " --output_mode %s" % output_mode if output_mode else ""
        #options += " -stand_emit_conf %i" % stand_emit_conf
        options += " -stand_call_conf %i" % stand_call_conf
        options += " --emitRefConfidence GVCF" if gvcf_mode else ""

        return options

    def parse_options(self, reference, alignment, output, genotyping_mode="DISCOVERY", output_mode="EMIT_VARIANTS_ONLY",
                      stand_call_conf=30, gvcf_mode=False, include_region_id_file=None, exclude_region_id_file=None):

        options = " -nct %i" % self.threads
        options += " -R %s" % reference
        options += " -I %s" % alignment
        options += " --genotyping_mode %s" % genotyping_mode if genotyping_mode else ""
        options += " --output_mode %s" % output_mode if output_mode else ""
        #options += " -stand_emit_conf %i" % stand_emit_conf
        options += " -stand_call_conf %i" % stand_call_conf
        options += " --emitRefConfidence GVCF" if gvcf_mode else ""
        options += " -L %s" % include_region_id_file if include_region_id_file else ""
        options += " -XL %s" % exclude_region_id_file if exclude_region_id_file else ""

        options += " -o %s" % output

        return options

    def call(self, reference, alignment, output, genotyping_mode="DISCOVERY", output_mode="EMIT_VARIANTS_ONLY",
             stand_call_conf=30, include_region_id_file=None, exclude_region_id_file=None):
        """
            java -Xmx100g -jar ~/tools/GenomeAnalysisTK-3.7/GenomeAnalysisTK.jar \
              -T HaplotypeCaller \
              -R ${fasta} \
              -I ${bam%bam}realigned.bam \
              --genotyping_mode DISCOVERY \
              --output_mode EMIT_VARIANTS_ONLY \
              -stand_call_conf 30 \
              -o ${bam%bam}raw.vcf
        """
        options = self.parse_options(reference, alignment, output, genotyping_mode=genotyping_mode,
                                     output_mode=output_mode,
                                     stand_call_conf=stand_call_conf, gvcf_mode=False,
                                     include_region_id_file=include_region_id_file,
                                     exclude_region_id_file=exclude_region_id_file)

        self.execute(options)

    def gvcf_call(self, reference, alignment, output, genotyping_mode="DISCOVERY",
                  stand_call_conf=30, include_region_id_file=None, exclude_region_id_file=None):
        """
        java -jar GenomeAnalysisTK.jar \
         -R reference.fasta \
         -T HaplotypeCaller \
         -I sample1.bam \
         --emitRefConfidence GVCF \
         [--dbsnp dbSNP.vcf] \
         [-L targets.interval_list] \
         -o output.raw.snps.indels.g.vcf
        """
        options = self.parse_options(reference, alignment, output, genotyping_mode=genotyping_mode,
                                     stand_call_conf=stand_call_conf, gvcf_mode=True,
                                     include_region_id_file=include_region_id_file,
                                     exclude_region_id_file=exclude_region_id_file)

        self.execute(options)

    def parallel_gvcf_call(self, reference, alignment, output_dir, output_prefix, output,
                           genotyping_mode="DISCOVERY",
                           stand_call_conf=30, max_region_length=1000000, max_seqs_per_region=100,
                           length_dict=None, parsing_mode="parse", region_list=None,
                           region_file_format='simple',
                           remove_intermediate_files=False,
                           gvcf_extension_list=["g.vcf", ],
                           cpus_per_task=1,
                           handling_mode="local",
                           job_name=None,
                           log_prefix=None,
                           error_log_prefix=None,
                           max_running_jobs=None,
                           max_running_time=None,
                           max_memmory_per_cpu=None,
                           modules_list=None,
                           environment_variables_dict=None):
        splited_dir = "%s/splited_gvcf/" % output_dir
        regions_dir = "%s/regions/" % output_dir

        for directory in output_dir, splited_dir:
            self.safe_mkdir(directory)

        region_list, \
            scaffold_to_region_correspondence_dict = self.prepare_region_list_by_length(max_length=max_region_length,
                                                                                        max_seq_number=max_seqs_per_region,
                                                                                        length_dict=length_dict,
                                                                                        reference=None if length_dict is not None else reference,
                                                                                        parsing_mode=parsing_mode,
                                                                                        output_dir=regions_dir,
                                                                                        region_file_format=region_file_format if handling_mode != "slurm" else 'GATK') if region_list is None else region_list

        options = self.parse_options_for_parallel_run(reference, alignment,
                                                      genotyping_mode=genotyping_mode,
                                                      stand_call_conf=stand_call_conf,
                                                      gvcf_mode=True)
        options += " -nct 1"
        options_list = []

        output_index = 1

        output_file_list = []

        if handling_mode == 'local':
            for regions in region_list:
                output_file = "%s/%s_%i.g.vcf" % (splited_dir, output_prefix, output_index)
                region_options = " -o %s" % output_file
                output_file_list.append(output_file)
                #for region in regions:
                #    region_options += " -L %s:%i-%i" % (region[0], region[1], region[2])

                for region in regions:
                    if isinstance(region, str):
                        region_options += " -L %s" % region
                    elif len(region) == 1:
                        region_options += " -L %s" % region[0]
                    elif len(region) == 3:
                        region_options += " -L %s:%i-%i" % (region[0], region[1], region[2])

                options_list.append(options + region_options)
                output_index += 1

            self.parallel_execute(options_list)

            VCFRoutines.combine_same_samples_vcfs(output,
                                                  vcf_list=output_file_list,
                                                  order_vcf_files=True,
                                                  close_fd_after=False,
                                                  extension_list=gvcf_extension_list)

        elif handling_mode == 'slurm':
            number_of_regions = len(region_list)
            region_file = "%s/splited/region_${SLURM_ARRAY_TASK_ID}.list" % regions_dir
            output_file = "%s/%s_${SLURM_ARRAY_TASK_ID}.g.vcf" % (splited_dir, output_prefix)
            options += " -o %s" % output_file
            options += " -L %s" % region_file

            slurm_cmd = "java -Xmx%s -jar %s/%s %s" % (self.max_memory, self.jar_path, self.jar, options)

            last_job_id = self.slurm_run_job(job_name,
                                             log_prefix,
                                             slurm_cmd,
                                             error_log_prefix,
                                             "%s%s.slurm" % (output_dir, output_prefix),
                                             task_index_list=None,
                                             start_task_index=1,
                                             end_task_index=number_of_regions,
                                             max_running_jobs=max_running_jobs,
                                             max_running_time=max_running_time,
                                             cpus_per_task=cpus_per_task,
                                             max_memmory_per_cpu=max_memmory_per_cpu,
                                             modules_list=modules_list,
                                             environment_variables_dict=environment_variables_dict)

            print("Submitted job  %s" % last_job_id)

            #VCFRoutines.combine_same_samples_vcfs(output_file_list,
            #                                      output,
            #                                      close_fd_after=False,
            #                                      extension_list=gvcf_extension_list)
        else:
            print("ERROR!!! Unrecognized handling mode!")

    def variant_call(self,
                     alignment,
                     reference_file,
                     stand_emit_conf=40,
                     stand_call_conf=100,
                     GATK_dir="",
                     num_of_threads=5,
                     output_mode="EMIT_VARIANTS_ONLY",
                     discovery_mode="BOTH",
                     output_file="GATK_raw.vcf",
                     default_base_qualities=None):
        """
        OBSOLETE FUNCTION!!! DOESN'T WORK
        """
        # manual http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_gatk_tools_walkers_genotyper_UnifiedGenotyper.html
        # output_mode values:
        # EMIT_VARIANTS_ONLY
        # EMIT_ALL_CONFIDENT_SITES
        # EMIT_ALL_SITES

        # discovery_mode values:
        # SNP
        # INDEL
        # GENERALPLOIDYSNP
        # GENERALPLOIDYINDEL
        # BOTH


        default_qualities = ""
        if default_base_qualities:
            default_qualities = "--defaultBaseQualities %i" % default_base_qualities

        gatk_dir = FileRoutines.check_path(GATK_dir)
        os.system(" java -jar %sGenomeAnalysisTK.jar -nt %i -l INFO -R %s -T HaplotypeCaller -I %s -stand_call_conf %i -stand_emit_conf %i -o %s --output_mode %s -glm %s %s"
                  % (gatk_dir, num_of_threads, reference_file, alignment, stand_call_conf, stand_emit_conf, output_file, output_mode, discovery_mode, default_qualities))


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Parsers.Barrnap import CollectionBARRNAP



class Barrnap(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "barrnap", path=path, max_threads=max_threads)
        self.kingdoms = ["euk", "bac", "mito", "arc"]

    def parse_options(self, input_fasta, kingdom, gff_file, log_file, length_cutoff=None, reject_cutoff=None,
                      evalue_cutoff=None):

        if kingdom not in self.kingdoms:
            raise ValueError("Wrong kingdom of life. Allowed: euk, bac, mito, arc")

        options = " --threads %i" % self.threads
        options += " --kingdom %s" % kingdom
        options += " --lencutoff %f" % length_cutoff if length_cutoff else ""
        options += " --reject %f" % reject_cutoff if reject_cutoff else ""
        options += " --evalue %f" % evalue_cutoff if evalue_cutoff else ""
        options += " %s" % input_fasta
        options += " > %s " % gff_file
        options += " 2> %s" % log_file

        return options

    def annotate_rrna(self, input_fasta, kingdom, output_prefix, length_cutoff=None, reject_cutoff=None,
                      evalue_cutoff=None):

        gff_file = "%s.%s.gff" % (output_prefix, kingdom)
        log_file = "%s.%s.log" % (output_prefix, kingdom)
        options = self.parse_options(input_fasta, kingdom, gff_file, log_file, length_cutoff=length_cutoff,
                                     reject_cutoff=reject_cutoff, evalue_cutoff=evalue_cutoff)
        self.execute(options)
        self.analyze_report(log_file, "%s.%s" % (output_prefix, kingdom))

        if kingdom == "euk":
            mito_gff_file = "%s.mito.gff" % output_prefix
            mito_log_file = "%s.mito.log" % output_prefix
            mito_options = self.parse_options(input_fasta, "mito", mito_gff_file, mito_log_file,
                                              length_cutoff=length_cutoff,
                                              reject_cutoff=reject_cutoff, evalue_cutoff=evalue_cutoff)

            self.execute(mito_options)
            self.analyze_report(mito_log_file, "%s.mito" % output_prefix)

    @staticmethod
    def analyze_report(report_file, output_prefix):
        count_file = "%s.chrom.count" % output_prefix
        total_file = "%s.total.count" % output_prefix

        report = CollectionBARRNAP(from_file=True, barrnap_file=report_file)

        count_dict, total_count_dict = report.count_types(output_file=count_file,
                                                          total_output_file=total_file,
                                                          return_mode="both")
        return count_dict, total_count_dict


if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.Abstract import Tool


class Cufflinks(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "cufflinks", path=path, max_threads=max_threads)

    def assembly_transcripts(self, alignment, output_dir=None, GTF=None, GTF_guide=None, mask_file=None,
                             frag_bias_correct=None, label=None, library_type=None, library_norm_method=None, seed=None,
                             frag_len_mean=None, frag_len_std_dev=None, max_mle_iterations=None,
                             num_frag_count_draws=None, num_frag_assign_draws=None, max_frag_multihits=None,
                             max_intron_length=None, min_frags_per_transfrag=None, overhang_tolerance=None,
                             max_bundle_length=None, max_bundle_frags=None, min_intron_length=None,
                             trim_3_avgcov_thresh=None, overlap_radius=None, three_overhang_tolerance=None,
                             intron_overhang_tolerance=None, min_isoform_fraction=None, pre_mrna_fraction=None,
                             junc_alpha=None, small_anchor_fraction=None, trim_3_dropoff_frac=None,
                             max_multiread_fraction=None, multi_read_correct=None, no_effective_length_correction=None,
                             no_length_correction=None, compatible_hits_norm=None, total_hits_norm=None,
                             no_faux_reads=None, verbose=None, quiet=None, no_update_check=None):

        options = " --num-threads %i" % self.threads
        options += " --output-dir %s" % output_dir if output_dir else ""
        options += " --GTF %s" % GTF if GTF else ""
        options += " --GTF-guide %s" % GTF_guide if GTF_guide else ""
        options += " --mask-file %s" % mask_file if mask_file else ""
        options += " --frag-bias-correct %s" % frag_bias_correct if frag_bias_correct else ""
        options += " --label %s" % label if label else ""
        options += " --library-type %s" % library_type if library_type else ""
        options += " --library-norm-method %s" % library_norm_method if library_norm_method else ""

        options += " --seed %i" % seed if seed else ""
        options += " --frag-len-mean %i" % frag_len_mean if frag_len_mean else ""
        options += " --frag-len-std-dev %i" % frag_len_std_dev if frag_len_std_dev else ""
        options += " --max-mle-iterations %i" % max_mle_iterations if max_mle_iterations else ""

        options += " --num-frag-count-draws %i" % num_frag_count_draws if num_frag_count_draws else ""
        options += " --num-frag-assign-draws %i" % num_frag_assign_draws if num_frag_assign_draws else ""
        options += " --max-frag-multihits %i" % max_frag_multihits if max_frag_multihits else ""
        options += " --max-intron-length %i" % max_intron_length if max_intron_length else ""
        options += " --min-frags-per-transfrag %i" % min_frags_per_transfrag if min_frags_per_transfrag else ""
        options += " --overhang-tolerance %i" % overhang_tolerance if overhang_tolerance else ""
        options += " --max-bundle-length %i" % max_bundle_length if max_bundle_length else ""
        options += " --max-bundle-frags %i" % max_bundle_frags if max_bundle_frags else ""
        options += " --min-intron-length %i" % min_intron_length if min_intron_length else ""
        options += " --trim-3-avgcov-thresh %i" % trim_3_avgcov_thresh if trim_3_avgcov_thresh else ""
        options += " --overlap-radius %i" % overlap_radius if overlap_radius else ""

        options += " --3-overhang-tolerance %i" % three_overhang_tolerance if three_overhang_tolerance else ""
        options += " --intron-overhang-tolerance %i" % intron_overhang_tolerance if intron_overhang_tolerance else ""

        options += " --min-isoform-fraction %f" % min_isoform_fraction if min_isoform_fraction else ""
        options += " --pre-mrna-fraction %f" % pre_mrna_fraction if pre_mrna_fraction else ""
        options += " --junc-alpha %f" % junc_alpha if junc_alpha else ""
        options += " --small-anchor-fraction %f" % small_anchor_fraction if small_anchor_fraction else ""
        options += " --trim-3-dropoff-frac %f" % trim_3_dropoff_frac if trim_3_dropoff_frac else ""
        options += " --max-multiread-fraction %f" % max_multiread_fraction if max_multiread_fraction else ""

        options += " --multi-read-correct" if multi_read_correct else ""
        options += " --no-effective-length-correction" if no_effective_length_correction else ""
        options += " --no-length-correction" if no_length_correction else ""
        options += " --compatible-hits-norm" if compatible_hits_norm else ""
        options += " --total-hits-norm" if total_hits_norm else ""
        options += " --no-faux-reads" if no_faux_reads else ""
        options += " --verbose" if verbose else ""
        options += " --quiet" if quiet else ""
        options += " --no-update-check" if no_update_check else ""

        options += " %s" % alignment

        self.execute(options=options)


class Cuffmerge(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "cuffmerge", path=path, max_threads=max_threads)

    def merge(self, gtf_list_file, output_dir="./", reference_gtf=None, reference_fasta=None, min_isoform_fraction=None, keep_tmp=None):

        options = " --num-threads %i" % self.threads
        options += " -o %s" % output_dir
        options += " --ref-gtf %s " % reference_gtf if reference_gtf else ""
        options += " --ref_sequence %s" % reference_fasta if reference_fasta else ""
        options += " --min-isoform-fraction %f" % min_isoform_fraction if min_isoform_fraction else ""
        options += " --keep-tmp" if keep_tmp else ""
        options += " %s" % gtf_list_file

        self.execute(options=options)


class Cuffquant(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "cuffquant", path=path, max_threads=max_threads)

    def quant(self, output_dir, gtf_file, sam_files, mask_file=None, frag_bias_correct=None, library_type=None, seed=None, quiet=None,
              verbose=None, frag_len_mean=None, frag_len_std_dev=None, max_mle_iterations=None, no_update_check=None,
              max_bundle_frags=None, max_frag_multihits=None, no_effective_length_correction=None,
              no_length_correction=None, no_scv_correction=None, read_skip_fraction=None, no_read_pairs=None,
              min_alignment_count=None):

        options = " --num-threads %i" % self.threads
        options += " --output-dir %s" % output_dir if output_dir else ""
        options += " --mask-file %s" % mask_file if mask_file else ""
        options += " --frag-bias-correct %s" % frag_bias_correct if frag_bias_correct else ""
        options += " --library-type %s" % library_type if library_type else ""
        options += " --seed %i" % seed if seed else ""
        options += " --frag-len-mean %i" % frag_len_mean if frag_len_mean else ""
        options += " --frag-len-std-dev %i" % frag_len_std_dev if frag_len_std_dev else ""
        options += " --max-mle-iterations %i" % max_mle_iterations if max_mle_iterations else ""
        options += " --max-frag-multihits %i" % max_frag_multihits if max_frag_multihits else ""
        options += " --max-bundle-frags %i" % max_bundle_frags if max_bundle_frags else ""
        options += " --min-alignment-count %i" % min_alignment_count if min_alignment_count else ""
        options += " --read-skip-fraction %f" % read_skip_fraction if read_skip_fraction else ""
        options += " --no-effective-length-correction" if no_effective_length_correction else ""
        options += " --no-length-correction" if no_length_correction else ""
        options += " --no-read-pairs" if no_read_pairs else ""
        options += " --no-scv-correction" if no_scv_correction else ""
        options += " --verbose" if verbose else ""
        options += " --quiet" if quiet else ""
        options += " --no-update-check" if no_update_check else ""

        options += " %s" % gtf_file
        options += " %s" % (sam_files if isinstance(sam_files, str) else " ".join(sam_files))

        self.execute(options=options)


class Cuffnorm(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "cuffnorm", path=path, max_threads=max_threads)

    def normalize(self, output_dir, gtf_file, cxb_files_two_lvl_list, labels_list=None, norm_standards_file=None,
                  library_norm_method=None, library_type=None, seed=None, quiet=None,
                  verbose=None, no_update_check=None, output_format=None,
                  compatible_hits_norm=None, total_hits_norm=None):

        options = " --num-threads %i" % self.threads
        options += " --output-dir %s" % output_dir if output_dir else ""
        options += " --output_format" % output_format if output_format else ""
        options += " --library-type %s" % library_type if library_type else ""
        options += " --norm_standards_file %s" % norm_standards_file if norm_standards_file else ""
        options += " --library-norm-method %s" % library_norm_method if library_norm_method else ""
        options += (" --labels %s" % ",".join(labels_list)) if labels_list else ""
        options += " --seed %i" % seed if seed else ""
        options += " --compatible-hits-norm" if compatible_hits_norm else ""
        options += " --total-hits-norm" if total_hits_norm else ""
        options += " --verbose" if verbose else ""
        options += " --no-update-check" if no_update_check else ""
        options += " --quiet" if quiet else ""
        options += " %s" % gtf_file
        options += " %s" % " ".join(map(lambda s: s if isinstance(s, str) else ",".join(s), cxb_files_two_lvl_list))

        self.execute(options=options)


class Cuffdiff(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "cuffdiff", path=path, max_threads=max_threads)

    def compare(self, gtf_file, sam_files_two_lvl_list, output_dir=None, mask_file=None, frag_bias_correct=None,
                library_type=None, library_norm_method=None, seed=None, frag_len_mean=None, frag_len_std_dev=None,
                max_mle_iterations=None, num_frag_count_draws=None, num_frag_assign_draws=None, max_frag_multihits=None,
                max_bundle_frags=None, multi_read_correct=None, no_effective_length_correction=None,
                no_length_correction=None, compatible_hits_norm=None, total_hits_norm=None, verbose=None, quiet=None,
                no_update_check=None, labels_list=None, dispersion_method=None, min_reps_for_js_test=None,
                no_scv_correction=None, min_alignment_count=None, read_skip_fraction=None, no_read_pairs=None,
                FDR=None, no_diff=None, no_js_tests=None, time_series=None
                ):

        options = " --num-threads %i" % self.threads
        options += " --output-dir %s" % output_dir if output_dir else ""
        options += (" --labels %s" % ",".join(labels_list)) if labels_list else ""
        options += " --mask-file %s" % mask_file if mask_file else ""
        options += " --multi-read-correct" if multi_read_correct else ""
        options += " --library-type %s" % library_type if library_type else ""
        options += " --dispersion-method %s" % dispersion_method if dispersion_method else ""

        options += " --library-norm-method %s" % library_norm_method if library_norm_method else ""
        options += " --frag-len-mean %i" % frag_len_mean if frag_len_mean else ""
        options += " --frag-len-std-dev %i" % frag_len_std_dev if frag_len_std_dev else ""

        options += " --max-bundle-frags %i" % max_bundle_frags if max_bundle_frags else ""
        options += " --min-reps-for-js-test %i" % min_reps_for_js_test if min_reps_for_js_test else ""

        options += " --compatible-hits-norm" if compatible_hits_norm else ""
        options += " --total-hits-norm" if total_hits_norm else ""

        options += " --no-scv-correction" if no_scv_correction else ""
        options += " --verbose" if verbose else ""
        options += " --quiet" if quiet else ""

        options += " --min-alignment-count %i" % min_alignment_count if min_alignment_count else ""
        options += " --read-skip-fraction %f" % read_skip_fraction if read_skip_fraction else ""
        options += " --no-effective-length-correction" if no_effective_length_correction else ""
        options += " --no-length-correction" if no_length_correction else ""

        options += " --num-frag-count-draws %i" % num_frag_count_draws if num_frag_count_draws else ""
        options += " --num-frag-assign-draws %i" % num_frag_assign_draws if num_frag_assign_draws else ""
        options += " --max-frag-multihits %i" % max_frag_multihits if max_frag_multihits else ""
        options += " --no-update-check" if no_update_check else ""
        options += " --seed %i" % seed if seed else ""
        options += " --max-mle-iterations %i" % max_mle_iterations if max_mle_iterations else ""
        options += " --no-read-pairs" if no_read_pairs else ""
        options += " --frag-bias-correct %s" % frag_bias_correct if frag_bias_correct else ""

        options += " --FDR %f" % FDR if FDR else ""
        options += " --no-diff" if no_diff else ""
        options += " --no-js-tests" if no_js_tests else ""
        options += " --time-series" if time_series else ""

        options += " %s" % gtf_file
        options += " %s" % " ".join(map(lambda s: s if isinstance(s, str) else ",".join(s), sam_files_two_lvl_list))

        self.execute(options=options)


class Cuffcompare(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "cuffcompare", path=path, max_threads=max_threads)

    def compare(self):

        options = ""
        self.execute(options=options)
        pass


class Gtf2sam(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "gtf_to_sam", path=path, max_threads=max_threads)

    def convert(self, gtf_list, output_sam, reference_fasta=None, use_fpkm_instead_isoform_fraction=False):

        options = " --reference-seq %s" % reference_fasta if reference_fasta else ""
        options += " --raw-fpkm" if use_fpkm_instead_isoform_fraction else ""
        options += " %s" % (gtf_list if isinstance(gtf_list, str) else ",".join(gtf_list))
        options += " %s" % output_sam


class Gffread(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "gffread", path=path, max_threads=max_threads)

    @staticmethod
    def parse_common_options(input_gff_file, genomic_fasta_file=None, output_cds_file=None, output_protein_file=None,
                             output_transcripts_file=None, coding_only=False):

        options = " %s" % input_gff_file
        options += " -C" if coding_only else ""
        options += " -g %s" % genomic_fasta_file if genomic_fasta_file else ""
        options += " -x %s" % output_cds_file if output_cds_file else ""
        options += " -w %s" % output_transcripts_file if output_transcripts_file else ""
        options += " -y %s" % output_protein_file if output_protein_file else ""
        return options

    def extract_transcript_sequences(self, input_gff_file, genomic_fasta_file, output_prefix, coding_only=False):

        output_cds_file = "%s.cds" % output_prefix
        output_transcripts_file = "%s.transcript" % output_prefix
        output_protein_file = "%s.pep" % output_prefix
        options = self.parse_common_options(input_gff_file,
                                            coding_only=coding_only,
                                            genomic_fasta_file=genomic_fasta_file,
                                            output_cds_file=output_cds_file,
                                            output_protein_file=output_protein_file,
                                            output_transcripts_file=output_transcripts_file)
        #print options
        self.execute(options=options)
        return output_transcripts_file, output_cds_file, output_protein_file

    def extract_cds(self, input_gff_file, genomic_fasta_file, output_file):

        options = self.parse_common_options(input_gff_file,
                                            genomic_fasta_file=genomic_fasta_file,
                                            output_cds_file=output_file,
                                            coding_only=True,
                                            output_transcripts_file=None)
        #print options
        self.execute(options=options)

    def extract_mRNA(self, input_gff_file, genomic_fasta_file, output_file, coding_only=False):

        options = self.parse_common_options(input_gff_file,
                                            coding_only=coding_only,
                                            genomic_fasta_file=genomic_fasta_file,
                                            output_transcripts_file=output_file)
        #print options
        self.execute(options=options)

if __name__ == "__main__":
    pass



import os
from collections import OrderedDict
import numpy as np
import matplotlib



matplotlib.use('Agg')
os.environ['MPLCONFIGDIR'] = '/tmp/'
import matplotlib.pyplot as plt
plt.ioff()

from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord

from RouToolPa.Tools.Abstract import Tool
from Parsers.PLINK import PLINKReport

from RouToolPa.Routines import MatplotlibRoutines


class PLINK(Tool):
    def __init__(self, path="", max_threads=4, max_memory="100G", max_per_thread_memory="5G"):
        Tool.__init__(self, "plink", path=path, max_threads=max_threads, max_memory=max_memory, max_per_thread_memory=max_per_thread_memory)

        # constants for PLINK bed file
        self.samples_per_byte = 4
        self.signature_byte_number = 3

        # constans for PLINK bim file
        self.allel_columns_in_bim_file = (4, 5)

    def parse_common_options(self, output_prefix, input_vcf_file=None, allow_noncanonical_chromosome_names=None, keep_autoconverted_files=None):

        options = " --threads %i" % self.threads
        options += " --out %s" % output_prefix
        options += " --allow-extra-chr" if allow_noncanonical_chromosome_names else ""
        options += " --keep-autoconv" if keep_autoconverted_files else ""
        options += " --vcf %s" % input_vcf_file if input_vcf_file else ""

        return options

    def find_runs_of_homozygosity_options(self,
                                          roh_calling_method=None,
                                          window_length_in_kb=None,
                                          min_homozygous_snps_per_window=None,
                                          max_heterozygous_snps_per_window=None,
                                          max_missing_snps_per_window=None,
                                          max_inverse_density_of_homozygous_snps_in_kb_per_snp=None,
                                          max_internal_gap_in_kb=None,
                                          min_roh_length=None,
                                          min_homozygous_snps_in_roh=None,
                                          min_scanning_window_hit_rate=None,
                                          generate_overlapping_segments=False,
                                          max_heterozygous_snps=None,
                                          min_concordance_across_jointly_homozygous_variants=None,
                                          homozygous_verbose=False):
        """
        ROH calling methods:
        <group | group-verbose> <consensus-match> <extend> <subtract-1-from-lengths>

        """
        options = " --homozyg %s" % roh_calling_method if roh_calling_method else " --homozyg"
        options += " --homozyg-window-snp %i" % min_homozygous_snps_per_window if min_homozygous_snps_per_window else ""                      # 50
        options += " --homozyg-window-het %i" % max_heterozygous_snps_per_window if max_heterozygous_snps_per_window else ""                  # 1
        options += " --homozyg-window-missing %i" % max_missing_snps_per_window if max_missing_snps_per_window else ""                        # 5
        options += " --homozyg-window-kb %i" % window_length_in_kb if window_length_in_kb else ""                                             # 5000
        options += " --homozyg-window-threshold %f" % min_scanning_window_hit_rate if min_scanning_window_hit_rate else ""                    # 0.05

        options += " --homozyg-density %i" % max_inverse_density_of_homozygous_snps_in_kb_per_snp if max_inverse_density_of_homozygous_snps_in_kb_per_snp else "" # 50
        options += " --homozyg-gap %i" % max_internal_gap_in_kb if max_internal_gap_in_kb else ""                                                                 # 1000
        options += " --homozyg-snp %i" % min_homozygous_snps_in_roh if min_homozygous_snps_in_roh else ""                                                         # 100
        options += " --homozyg-kb %i" % min_roh_length if min_roh_length else ""                                                                                  # 1000
        options += " --homozyg-match %f" % min_concordance_across_jointly_homozygous_variants if min_concordance_across_jointly_homozygous_variants else ""       # 0.95
        options += " --homozyg-het %i" % max_heterozygous_snps if max_heterozygous_snps else ""                                                                 # unlimited
        options += " --homozyg-group" if generate_overlapping_segments else ""
        options += " --homozyg-verbose" if homozygous_verbose else "" #

        return options

    def find_runs_of_homozygosity(self,
                                  output_prefix,
                                  input_vcf_file=None,
                                  allow_noncanonical_chromosome_names=None,
                                  keep_autoconverted_files=None,
                                  roh_calling_method=None,
                                  window_length_in_kb=None,
                                  min_homozygous_snps_per_window=None,
                                  max_heterozygous_snps_per_window=None,
                                  max_missing_snps_per_window=None,
                                  max_inverse_density_of_homozygous_snps_in_kb_per_snp=None,
                                  max_internal_gap_in_kb=None,
                                  min_roh_length=None,
                                  min_homozygous_snps_in_roh=None,
                                  min_scanning_window_hit_rate=None,
                                  generate_overlapping_segments=False,
                                  max_heterozygous_snps=None,
                                  min_concordance_across_jointly_homozygous_variants=None,
                                  homozygous_verbose=False):
        options = self.parse_common_options(output_prefix,
                                            input_vcf_file=input_vcf_file,
                                            allow_noncanonical_chromosome_names=allow_noncanonical_chromosome_names,
                                            keep_autoconverted_files=keep_autoconverted_files)

        options += self.find_runs_of_homozygosity_options(roh_calling_method=roh_calling_method,
                                                          window_length_in_kb=window_length_in_kb,
                                                          min_homozygous_snps_per_window=min_homozygous_snps_per_window,
                                                          max_heterozygous_snps_per_window=max_heterozygous_snps_per_window,
                                                          max_missing_snps_per_window=max_missing_snps_per_window,
                                                          max_inverse_density_of_homozygous_snps_in_kb_per_snp=max_inverse_density_of_homozygous_snps_in_kb_per_snp,
                                                          max_internal_gap_in_kb=max_internal_gap_in_kb,
                                                          min_roh_length=min_roh_length,
                                                          min_homozygous_snps_in_roh=min_homozygous_snps_in_roh,
                                                          min_scanning_window_hit_rate=min_scanning_window_hit_rate,
                                                          generate_overlapping_segments=generate_overlapping_segments,
                                                          max_heterozygous_snps=max_heterozygous_snps,
                                                          min_concordance_across_jointly_homozygous_variants=min_concordance_across_jointly_homozygous_variants,
                                                          homozygous_verbose=homozygous_verbose)
        self.execute(options=options)

    def test_roh_parameters(self,
                            output_dir,
                            output_prefix,
                            input_vcf_file,
                            allow_noncanonical_chromosome_names=True,
                            keep_autoconverted_files=None,
                            window_length_in_kb=None,
                            min_homozygous_snps_per_window=(2, 51, 1),
                            min_homozygous_snps_in_roh=(2, 101, 1),
                            max_heterozygous_snps_per_window=(1, 11, 1),
                            max_heterozygous_snps=(1, 21, 1),
                            max_inverse_density_of_homozygous_snps_in_kb_per_snp=(50, 1000, 50),
                            ):
        self.safe_mkdir(output_dir)

        plink_report_dict = OrderedDict()
        roh_count_array = np.zeros((len(range(*min_homozygous_snps_per_window)),
                                    len(range(*max_heterozygous_snps_per_window)),
                                    len(range(*min_homozygous_snps_in_roh)),
                                    len(range(*max_heterozygous_snps)),
                                    len(range(*max_inverse_density_of_homozygous_snps_in_kb_per_snp)),
                                    ), dtype=int)

        i_ticks = range(*min_homozygous_snps_per_window)
        j_ticks = range(*max_heterozygous_snps_per_window)
        k_ticks = range(*min_homozygous_snps_in_roh)
        l_ticks = range(*max_heterozygous_snps)
        m_ticks = range(*max_inverse_density_of_homozygous_snps_in_kb_per_snp)

        for i in i_ticks:
            plink_report_dict[i] = OrderedDict()
            for j in j_ticks:
                plink_report_dict[i][j] = OrderedDict()
                for k in k_ticks:
                    plink_report_dict[i][j][k] = OrderedDict()
                    for l in l_ticks:
                        plink_report_dict[i][j][k][l] = OrderedDict()
                        for m in m_ticks:
                            dir_name = "%s/%i_%i_%i_%i_%i/" % (output_dir, i, j, k, l, m)
                            description_text = "Minimum homozygous SNPs per window:\t%i\n" % i
                            description_text += "Minimum homozygous SNPs in ROh:\t%i\n" % k
                            description_text += "Maximum heterozygous SNPs per window:\t%i\n" % j
                            description_text += "Max heterozygous SNPs:\t%i\n" % l
                            description_text += "Max inverse density of homozygous SNPs(kb/SNP):\t%i\n" % m
                            self.safe_mkdir(dir_name, description_text=description_text, description_filename="DESCRIPTION")
                            self.find_runs_of_homozygosity("%s/%s" % (dir_name, output_prefix),
                                                           input_vcf_file=input_vcf_file,
                                                           allow_noncanonical_chromosome_names=allow_noncanonical_chromosome_names,
                                                           keep_autoconverted_files=keep_autoconverted_files,
                                                           roh_calling_method=None,
                                                           window_length_in_kb=window_length_in_kb,
                                                           min_homozygous_snps_per_window=i,
                                                           max_heterozygous_snps_per_window=j,
                                                           max_missing_snps_per_window=None,
                                                           max_inverse_density_of_homozygous_snps_in_kb_per_snp=m,
                                                           max_internal_gap_in_kb=None,
                                                           min_roh_length=None,
                                                           min_homozygous_snps_in_roh=k,
                                                           min_scanning_window_hit_rate=None,
                                                           generate_overlapping_segments=False,
                                                           max_heterozygous_snps=l,
                                                           min_concordance_across_jointly_homozygous_variants=None,
                                                           homozygous_verbose=False)
                            plink_report_dict[i][j][k][l][m] = PLINKReport("%s/%s.hom" % (dir_name, output_prefix),
                                                                           report_type="ROH")
                            roh_count_array[i_ticks.index(i)][j_ticks.index(j)][k_ticks.index(k)][l_ticks.index(l)][m_ticks.index(m)] = len(plink_report_dict[i][j][k][l][m])
        figure_dir = "%s/pic/" % output_dir
        self.safe_mkdir(figure_dir)

        num_k_ticks = len(k_ticks)
        num_l_ticks = len(l_ticks)

        for m in m_ticks:
            figure, subplot_list = plt.subplots(num_k_ticks, num_l_ticks, sharex=True, sharey=True)
            plt.suptitle("Number of ROH depending on several parameters")
            for subplot_index in range(0, len(subplot_list)):
                k = int(subplot_index / num_l_ticks)
                l = subplot_index % num_l_ticks

                roh_counts = roh_count_array[:, :, k, l, m]

                title = "Max heterozygous SNPs: %i" % l_ticks[l] if k == 0 else None
                xlabel = "Min homozygous SNPs per window" if k == num_k_ticks - 1 else None
                ylabel = "" if l == 0 else None
                image, colorbar = MatplotlibRoutines.annotated_heatmap(roh_counts, i_ticks, j_ticks, subplot=subplot_list[subplot_index],
                                                                       title=title, xlabel=xlabel, ylabel=None)
                """
                heatmap = subplot_list[subplot_index].imshow(roh_counts)
                subplot_list[subplot_index].set_xticks(np.arange(len(i_ticks)))
                subplot_list[subplot_index].set_yticks(np.arange(len(j_ticks)))

                subplot_list[subplot_index].set_xticklabels(i_ticks)
                subplot_list[subplot_index].set_yticklabels(j_ticks)

                colorbar = subplot_list[subplot_index].figure.colorbar(heatmap, ax=subplot_list[subplot_index])
                """

            plt.savefig("%s/%i.png" % figure_dir, m)

    @staticmethod
    def get_samples_list_from_plink_fam_file(plink_fam_file, verbose=False, sample_id_column=1):
        samples_list = []
        with open(plink_fam_file, "r") as plink_fam_fd:
            for line in plink_fam_fd:
                samples_list.append(line.split("\t")[sample_id_column])

        if verbose:
            print("%i samples were found" % len(samples_list))

        return samples_list

    def extract_sequences_from_plink_binary_snp_data(self, plink_binary_files_prefix, output_file, verbose=False,
                                                     output_format="fasta"):

        plink_bed_file = "%s.bed" % plink_binary_files_prefix
        plink_bim_file = "%s.bim" % plink_binary_files_prefix
        plink_fam_file = "%s.fam" % plink_binary_files_prefix

        if verbose:
            print("Parsing .fam file...")
        samples_list = self.get_samples_list_from_plink_fam_file(plink_fam_file, verbose=verbose)
        sample_number = len(samples_list)

        bytes_per_snp = sample_number / self.samples_per_byte if sample_number % self.samples_per_byte == 0 else int(sample_number/self.samples_per_byte) + 1

        if verbose:
            print("Parsing .bim file...")
        allels_array = np.loadtxt(plink_bim_file, usecols=self.allel_columns_in_bim_file, dtype=np.str)
        snp_number = len(allels_array)

        if verbose:
            print("Reading .bed file...")
        with open(plink_bed_file, 'rb') as plink_bed_fd:
            plink_bed_fd.read(self.signature_byte_number)
            genotypes_array = np.reshape(np.fromfile(plink_bed_fd, dtype='uint8'), (-1, bytes_per_snp))

        if snp_number != len(genotypes_array):
            raise ValueError("ERROR!!! .bed and .bim files are incompatible by length or files were parsed with errors!")

        record_list = []

        for sample_index in range(0, sample_number):
            if verbose:
                print("Handling sample %i (%s)..." % (sample_index + 1, samples_list[sample_index]))
            sample_byte = int(sample_index / self.samples_per_byte)
            sample_offset = sample_index % self.samples_per_byte

            sample_genotype_list = []
            for pos_index in xrange(0, snp_number):
                pos_genotype = (genotypes_array[pos_index, sample_byte] >> (2 * sample_offset)) & 0b11
                #print pos_index
                #print allels_array[pos_index]
                if pos_genotype == 0b00:
                    #print allels_array[pos_genotype][0]
                    sample_genotype_list.append(allels_array[pos_index][0])

                elif pos_genotype == 0b01:
                    #print "N"
                    sample_genotype_list.append("N")
                elif pos_genotype == 0b10:
                    sample_genotype_list.append(self.ambiguous_nucleotides_string_reverse_dict[allels_array[pos_index][0] + allels_array[pos_index][1]
                                                                                          if allels_array[pos_index][0] < allels_array[pos_index][1]
                                                                                          else allels_array[pos_index][1] + allels_array[pos_index][0]])
                    #print self.ambiguous_nucleotides_string_reverse_dict[allels_array[pos_index][0] + allels_array[pos_index][1]
                    #                                                                      if allels_array[pos_index][0] < allels_array[pos_index][1]
                    #                                                                      else allels_array[pos_index][1] + allels_array[pos_index][0]]
                else:
                    sample_genotype_list.append(allels_array[pos_index][1])
                    #print allels_array[pos_index][1]

            record_list.append(SeqRecord(id=samples_list[sample_index],
                                         description="",
                                         seq=Seq("".join(sample_genotype_list))))
            #if verbose:
            #    print record_list[sample_index]

        SeqIO.write(record_list, output_file, format=output_format)

    def convert_blocks_det_file_to_gff(self, list_of_det_files, output_file,
                                       output_format="gff", haploblock_id_prefix="HB"):

        list_of_files = self.make_list_of_path_to_files(list_of_det_files)

        with open(output_file, "w") as out_fd:
            for filename in list_of_files:
                with open(filename, "r") as in_fd:
                    in_fd.readline()
                    block_index = 1
                    for line in in_fd:
                        tmp = line.strip().split()
                        if output_format == "gff":
                            out_fd.write("%s\tPLINK\thaploblock\t%s\t%s\t.\t.\t.\tID=%s%i;NSNPS=%s;SNPS=%s\n" % (tmp[0], tmp[1], tmp[2], haploblock_id_prefix, block_index, tmp[4], tmp[5]))
                        elif output_format == "bed":
                            out_fd.write("%s\t%s\t%s\t%s%i\t%s\t%s\n" % (tmp[0], tmp[1], tmp[2], haploblock_id_prefix, block_index, tmp[4], tmp[5]))
                        block_index += 1






#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

from RouToolPa.Tools.GapClosers.Sealer import Sealer
from RouToolPa.Tools.GapClosers.GapFiller import GapFiller

Sealer = Sealer()
GapFiller = GapFiller()



#!/usr/bin/env python

from RouToolPa.Tools.RepeatMasking.RepeatScout import RepeatScout
from RouToolPa.Tools.RepeatMasking.TRF import TRF
from RouToolPa.Tools.RepeatMasking.RepeatMasker import RepeatMasker
from RouToolPa.Tools.RepeatMasking.RepeatModeler import RepeatModeler


TRF = TRF()
RepeatScout = RepeatScout()
RepeatMasker = RepeatMasker()
RepeatModeler = RepeatModeler()



#!/usr/bin/env python
import pandas
__author__ = 'mahajrod'
from collections import OrderedDict
from RouToolPa.Collections.General import TwoLvlDict


class PLINKReport():
    def __init__(self, plink_report_file, report_type="ROH", separator="\s+"):
        #OrderedDict.__init__(self)
        self.report_type = report_type
        self.families = None
        self.samples = None
        self.roh_lengths = None

        index_col = None
        if report_type == "ROH":
            index_col = (0, 1)

        self.data = pandas.read_table(plink_report_file, sep=separator, index_col=index_col)

        if report_type == "ROH":
            self.families = self.data.index.levels[0]
            self.samples = self.data.index.levels[1]
        """
        with open(plink_report_file, "r") as in_fd:
            if report_type == "ROH":


                header_list = in_fd.readline().strip().split()

            for line in in_fd:
                tmp = line.strip().split("\t")
                if tmp[0] not in self:
                    self[tmp[0]] = OrderedDict()
                if tmp[1] not in self[tmp[0]]:
                    self[tmp[0]][tmp[1]] = OrderedDict()
                for i in range(2, len(header_list)):
                    self[tmp[0]][tmp[1]][header_list[i]] = tmp[i]
                for j in 2, 8, 10, 11, 12:
                    self[tmp[0]][tmp[1]][header_list[j]] = float(self[tmp[0]][tmp[1]][header_list[j]])
                for j in 6, 7, 9:
                    self[tmp[0]][tmp[1]][header_list[j]] = int(self[tmp[0]][tmp[1]][header_list[j]])
            """
    def __len__(self):
        return len(self.data)

    def get_roh_lengths(self, save=False):
        """
        return: pandas Series
        """
        if save:
            self.roh_lengths = self.data['POS2'] - self.data['POS1'] + 1
        return self.roh_lengths if self.roh_lengths is not None else self.data['POS2'] - self.data['POS1'] + 1


class PLINKReportCollection(OrderedDict):
    def __init__(self, report_dict=None, file_dict=None, report_type=None):
        OrderedDict.__init__(self)
        self.report_type = report_type
        if report_dict:
            for report_id in report_dict:
                self[report_id] = report_dict
        if file_dict:
            for file_id in file_dict:
                if file_id in self:
                    raise KeyError("Report with same id(%s) is already present in PLINKReportCollection" % str(file_id))
                self[file_id] = PLINKReport(file_dict[file_id])

    def get_general_stats(self):
        pass

    def write_general_stats(self, output_file):
        stat_dict = self.get_general_stats()
        stat_dict.write(output_file, absent_symbol=".", sort=False)



#!/usr/bin/env python
__author__ = 'mahajrod'
import numpy as np
from RouToolPa.Collections.General import IdList, SynDict


class ExpressionRoutines:
    def __init__(self):
        pass

    @staticmethod
    def divide_counts_by_base_level(input_file, output_prefix, base_level,
                                    separator="\t", verbose=True, secondary_base_lvl=None):
        output_file = "%s.divided_by_baselvl" % output_prefix
        zero_base_lvl_list = IdList()
        zero_both_base_lvls_list = IdList()
        zero_base_lvl_list_file = "%s.zero_base_lvl.ids" % output_prefix
        zero_both_base_lvls_list_file = "%s.zero_base_lvls.ids" % output_prefix
        with open(input_file, "r") as in_fd:
            header = in_fd.readline()
            header_list = header.strip().split(separator)
            data_base_level_index = header_list.index(base_level) - 1
            if secondary_base_lvl:
                data_secondary_base_level_index = header_list.index(secondary_base_lvl) - 1
            with open(output_file, "w") as out_fd:
                out_fd.write(header)
                for line in in_fd:
                    tmp_line = line.strip().split(separator)
                    data = np.array(map(float, tmp_line[1:]))
                    if data[data_base_level_index] == 0:
                        zero_base_lvl_list.append(tmp_line[0])
                        if not secondary_base_lvl:
                            if verbose:
                                print("Zero base level(%s) for %s...Skipping..." % (base_level, tmp_line[0]))
                            continue
                    if secondary_base_lvl:
                        if data[data_secondary_base_level_index] == 0:
                            zero_both_base_lvls_list.append(tmp_line[0])
                            if verbose:
                                print("Both base levels are zero (%s, %s) for %s...Skipping..." % (base_level,
                                                                                                   secondary_base_lvl,
                                                                                                   tmp_line[0]))
                            continue

                        data /= data[data_base_level_index] if data[data_base_level_index] != 0 else data[data_secondary_base_level_index]
                    else:
                        data /= data[data_base_level_index]
                    output_string = tmp_line[0] + "\t"
                    output_string += "\t".join(map(str, data))
                    output_string += "\n"
                    out_fd.write(output_string)

        zero_base_lvl_list.write(zero_base_lvl_list_file)
        zero_both_base_lvls_list.write(zero_both_base_lvls_list_file)

    @staticmethod
    def divide_counts_by_max_level(input_file, output_prefix,
                                   separator="\t", verbose=True,):
        output_file = "%s.divided_by_maxlvl" % output_prefix
        zero_max_lvl_list = IdList()

        zero_max_lvl_list_file = "%s.zero_max_lvl.ids" % output_prefix

        with open(input_file, "r") as in_fd:
            header = in_fd.readline()
            header_list = header.strip().split(separator)
            with open(output_file, "w") as out_fd:
                out_fd.write(header)
                for line in in_fd:
                    tmp_line = line.strip().split(separator)
                    data = np.array(map(float, tmp_line[1:]))
                    max_level = max(data)
                    if max_level == 0:
                        zero_max_lvl_list.append(tmp_line[0])

                        if verbose:
                            print("Zero max level for %s...Skipping..." % tmp_line[0])
                        continue

                    data /= max_level
                    output_string = tmp_line[0] + "\t"
                    output_string += "\t".join(map(str, data))
                    output_string += "\n"
                    out_fd.write(output_string)

        zero_max_lvl_list.write(zero_max_lvl_list_file)

    @staticmethod
    def extract_counts_by_max_level(input_file, output_prefix,
                                   separator="\t", verbose=True):
        output_file = "%s.divided_by_maxlvl" % output_prefix
        zero_max_lvl_list = IdList()

        zero_max_lvl_list_file = "%s.zero_max_lvl.ids" % output_prefix

        with open(input_file, "r") as in_fd:
            header = in_fd.readline()
            header_list = header.strip().split(separator)
            with open(output_file, "w") as out_fd:
                out_fd.write(header)
                for line in in_fd:
                    tmp_line = line.strip().split(separator)
                    data = np.array(map(float, tmp_line[1:]))
                    max_level = max(data)
                    if max_level == 0:
                        zero_max_lvl_list.append(tmp_line[0])

                        if verbose:
                            print("Zero max level for %s...Skipping..." % tmp_line[0])
                        continue

                    data /= max_level
                    output_string = tmp_line[0] + "\t"
                    output_string += "\t".join(map(str, data))
                    output_string += "\n"
                    out_fd.write(output_string)

        zero_max_lvl_list.write(zero_max_lvl_list_file)

    @staticmethod
    def divide_counts_by_several_base_level(input_file, output_prefix, base_levels,
                                            separator="\t", verbose=True,
                                            max_ratio_to_base_lvl=0.5):
        output_file = "%s.divided_by_max_baselvl" % output_prefix
        max_ratio_to_base_lvl_file = "%s.divided_by_max_baselvl.max_%f_ratio" % (output_prefix, max_ratio_to_base_lvl)
        zero_max_base_lvl_list = IdList()
        zero_max_base_lvl_list_file = "%s.zero_base_lvls.ids" % output_prefix
        max_ratio_to_base_lvl_fd = open(max_ratio_to_base_lvl_file, "w")
        with open(input_file, "r") as in_fd:
            header = in_fd.readline()
            header_list = header.strip().split(separator)

            data_base_lvl_index_list = []
            base_level_list = [base_levels] if isinstance(base_levels, str) else base_levels
            for level in base_level_list:
                data_base_lvl_index_list.append(header_list.index(level) - 1)

            with open(output_file, "w") as out_fd:
                out_fd.write(header)
                max_ratio_to_base_lvl_fd.write(header)
                for line in in_fd:
                    tmp_line = line.strip().split(separator)
                    data = np.array(map(float, tmp_line[1:]))
                    max_base_lvl = max(np.take(data, data_base_lvl_index_list))
                    if max_base_lvl == 0:
                        zero_max_base_lvl_list.append(tmp_line[0])
                        if verbose:
                            print("Zero max base level(s) for %s...Skipping..." % tmp_line[0])
                        continue

                    data /= max_base_lvl
                    output_string = tmp_line[0] + "\t"
                    output_string += "\t".join(map(str, data))
                    output_string += "\n"
                    if max(np.delete(data, data_base_lvl_index_list)) <= max_ratio_to_base_lvl:
                        max_ratio_to_base_lvl_fd.write(output_string)
                    out_fd.write(output_string)

        zero_max_base_lvl_list.write(zero_max_base_lvl_list_file)
        max_ratio_to_base_lvl_fd.close()

    @staticmethod
    def calculate_fpkm_for_count_table(count_table_file, transcript_length_file, output_file,
                                       separator="\t"):
        length_dict = SynDict(filename=transcript_length_file, expression=int, comments_prefix="#")

        with open(count_table_file, "r") as in_fd:
            header_list = in_fd.readline().strip().split(separator)

            samples_list = header_list[1:]
            gene_list = IdList()
            count_list = []
            for line in in_fd:
                tmp = line.strip().split(separator)
                gene_list.append(tmp[0])
                count_list.append(map(float, tmp[1:]))

            per_sample_total_counts = []

            for sample_index in range(0, len(samples_list)):
                total_counts = 0
                for gene_index in range(0, len(count_list)):
                    total_counts += count_list[gene_index][sample_index]
                per_sample_total_counts.append(total_counts)

        with open(output_file, "w") as out_fd:
            out_fd.write(separator.join(header_list) + "\n")
            for gene_index in range(0, len(count_list)):
                normalized_counts_list = []
                for sample_index in range(0, len(samples_list)):
                    gene_count = count_list[gene_index][sample_index] * (10**9) / length_dict[gene_list[gene_index]] / per_sample_total_counts[sample_index]
                    normalized_counts_list.append(gene_count)
                out_fd.write("%s\t%s\n" % (gene_list[gene_index], "\t".join(map(str, normalized_counts_list))))



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Primer3(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "primer3_core", path=path, max_threads=max_threads)

    @staticmethod
    def parse_options(input_file, output_file=None, settings_file=None, format_output=None,
                      strict_tags=True, error_file=None):

        options = " --format_output" if format_output else ""
        options += " --strict_tags" if strict_tags else ""
        options += " --p3_settings_file=%s" % settings_file if settings_file else ""
        options += " --error=%s" % error_file if error_file else ""
        options += " --output=%s" % output_file if output_file else ''

        options += " %s" % input_file

        return options

    def generate_config(self, primer_task="generic",
                        pick_left_primer=True, pick_right_primer=True, pick_internal_oligo=False,
                        optimal_primer_len=None, min_primer_len=None, max_primer_len=None, max_ns_accepted=None,
                        pcr_product_size_range=None, explain_primers=True, report_all_primers=True,
                        softmasked_input=False, optimal_GC=None, min_GC=None, max_GC=None,
                        optimal_melting_temperature=None, min_melting_temperature=None,
                        max_melting_temperature=None, black_list_of_seqs_fasta=None, mask_sequence=False,
                        directory_with_kmer_counts=None, kmer_file_prefix=None, thermodynamic_parameters_dir=None):
        """
        PRIMER_TASK:  generic,
                      check_primers,
                      pick_primer_list,
                      pick_cloning_primers,
                      pick_discriminative_primers

        TODO: A lot of PRIMER options were not implemented!!!
        """
        config = ""
        config += "PRIMER_TASK=%s\n" % primer_task if primer_task else ""
        config += "PRIMER_PICK_LEFT_PRIMER=%i\n" % (1 if pick_left_primer else 0)
        config += "PRIMER_PICK_INTERNAL_OLIGO=%i\n" % (1 if pick_internal_oligo else 0)
        config += "PRIMER_PICK_RIGHT_PRIMER=%i\n" % (1 if pick_right_primer else 0)

        config += "PRIMER_OPT_SIZE=%i\n" % optimal_primer_len if optimal_primer_len else ""
        config += "PRIMER_MIN_SIZE=%i\n" % min_primer_len if min_primer_len else ""
        config += "PRIMER_MAX_SIZE=%i\n" % max_primer_len if max_primer_len else ""

        config += "PRIMER_MAX_NS_ACCEPTED=%i\n" % max_ns_accepted if max_ns_accepted else ""
        config += "PRIMER_PRODUCT_SIZE_RANGE=%s\n" % ("-".join(map(str, pcr_product_size_range))) if pcr_product_size_range else ""

        config += "PRIMER_EXPLAIN_FLAG=1\n" if explain_primers else ""
        config += "P3_FILE_FLAG=1\n" if report_all_primers else ""
        config += "PRIMER_LOWERCASE_MASKING=1\n" if softmasked_input else ""

        config += "PRIMER_OPT_GC_PERCENT=%f\n" % optimal_GC if optimal_GC else ""
        config += "PRIMER_MIN_GC=%f\n" % min_GC if min_GC else ""
        config += "PRIMER_MAX_GC=%f\n" % max_GC if max_GC else ""

        config += "PRIMER_OPT_TM=%f\n" % optimal_melting_temperature if optimal_melting_temperature else ""
        config += "PRIMER_MIN_TM=%f\n" % min_melting_temperature if min_melting_temperature else ""
        config += "PRIMER_MAX_TM=%f\n" % max_melting_temperature if max_melting_temperature else ""

        config += "PRIMER_MISPRIMING_LIBRARY=%s\n" % black_list_of_seqs_fasta if black_list_of_seqs_fasta else ""
        config += "PRIMER_MASK_TEMPLATE=1\n" if mask_sequence else ""

        config += "PRIMER_MASK_KMERLIST_PATH=%s\n" % self.check_dir_path(directory_with_kmer_counts) if directory_with_kmer_counts else ""
        config += "PRIMER_MASK_KMERLIST_PREFIX=%s\n" % kmer_file_prefix if kmer_file_prefix else ""
        config += "PRIMER_THERMODYNAMIC_PARAMETERS_PATH=%s\n" % thermodynamic_parameters_dir if thermodynamic_parameters_dir else ""

        return config

    def write_config(self, config_file, primer_task="generic", pick_left_primer=True, pick_right_primer=True,
                     pick_internal_oligo=False, optimal_primer_len=None, min_primer_len=None, max_primer_len=None,
                     max_ns_accepted=None, pcr_product_size_range=None, explain_primers=True, report_all_primers=True,
                     softmasked_input=False, optimal_GC=None, min_GC=None, max_GC=None,
                     optimal_melting_temperature=None, min_melting_temperature=None,
                     max_melting_temperature=None, black_list_of_seqs_fasta=None, mask_sequence=False,
                     directory_with_kmer_counts=None, kmer_file_prefix=None,
                     thermodynamic_parameters_dir=None):

        config = "Primer3 File - http://primer3.sourceforge.net\n"
        config += "P3_FILE_TYPE=settings\n"
        config += "\n"
        config += "P3_FILE_ID=STR primers\n"

        config += self.generate_config(primer_task=primer_task, pick_left_primer=pick_left_primer,
                                       pick_right_primer=pick_right_primer, pick_internal_oligo=pick_internal_oligo,
                                       optimal_primer_len=optimal_primer_len, min_primer_len=min_primer_len,
                                       max_primer_len=max_primer_len, max_ns_accepted=max_ns_accepted,
                                       pcr_product_size_range=pcr_product_size_range, explain_primers=explain_primers,
                                       report_all_primers=report_all_primers, softmasked_input=softmasked_input,
                                       optimal_GC=optimal_GC, min_GC=min_GC, max_GC=max_GC,
                                       optimal_melting_temperature=optimal_melting_temperature,
                                       min_melting_temperature=min_melting_temperature,
                                       max_melting_temperature=max_melting_temperature,
                                       black_list_of_seqs_fasta=black_list_of_seqs_fasta,
                                       mask_sequence=mask_sequence,
                                       directory_with_kmer_counts=directory_with_kmer_counts,
                                       kmer_file_prefix=kmer_file_prefix,
                                       thermodynamic_parameters_dir=thermodynamic_parameters_dir)
        config += "=\n"

        with open(config_file, "w") as config_fd:
            config_fd.write(config)

    @staticmethod
    def generate_input_record(sequence_id, sequence,
                              target_list=None, excluded_region_list=None, included_region_list=None,
                              internal_oligo_excluded_region_list=None, overlap_junction_list=None,
                              forward_primer=None, reverse_primer=None, internal_oligo=None,
                              force_forward_start=None, force_forward_end=None,
                              force_reverse_start=None, force_reverse_end=None,
                              config=""):
        """
        TODO:
            NOT IMPLEMENTED
                    SEQUENCE_PRIMER_PAIR_OK_REGION_LIST (semicolon separated list of integer "quadruples"; default empty)

                        This tag allows detailed specification of possible locations of left and right primers in primer pairs.

                        The associated value must be a semicolon-separated list of

                        <left_start>,<left_length>,<right_start>,<right_length>
                        quadruples. The left primer must be in the region specified by <left_start>,<left_length> and the right primer must be in the region specified by <right_start>,<right_length>. <left_start> and <left_length> specify the location of the left primer in terms of the index of the first base in the region and the length of the region. <right_start> and <right_length> specify the location of the right primer in analogous fashion. As seen in the example below, if no integers are specified for a region then the location of the corresponding primer is not constrained.

                        Example:

                            SEQUENCE_PRIMER_PAIR_OK_REGION_LIST=100,50,300,50 ; 900,60,, ; ,,930,100

                    SEQUENCE_START_CODON_POSITION (int; default -2000000)

                        This parameter should be considered EXPERIMENTAL at this point. Please check the output carefully; some erroneous inputs might cause an error in Primer3.

                        Index of the first base of a start codon. This parameter allows Primer3 to select primer pairs to create in-frame amplicons e.g. to create a template for a fusion protein. Primer3 will attempt to select an in-frame left primer, ideally starting at or to the left of the start codon, or to the right if necessary. Negative values of this parameter are legal if the actual start codon is to the left of available sequence. If this parameter is non-negative Primer3 signals an error if the codon at the position specified by this parameter is not an ATG. A value less than or equal to -10^6 indicates that Primer3 should ignore this parameter.

                        Primer3 selects the position of the right primer by scanning right from the left primer for a stop codon. Ideally the right primer will end at or after the stop codon.

                    SEQUENCE_QUALITY (space separated integers; default empty)

                        A list of space separated integers. There must be exactly one integer for each base in SEQUENCE_TEMPLATE if this argument is non-empty. For example, for the sequence ANNTTCA... SEQUENCE_QUALITY might be 45 10 0 50 30 34 50 67 .... High numbers indicate high confidence in the base called at that position and low numbers indicate low confidence in the base call at that position. This parameter is only relevant if you are using a base calling program that provides quality information (for example phred).
        """

        input_record = "SEQUENCE_ID=%s\n" % sequence_id
        input_record += "SEQUENCE_TEMPLATE=%s\n" % str(sequence)
        input_record += "SEQUENCE_TARGET=%s\n" % (" ".join(map(lambda s: "%s,%s" % (str(s[0]), str(s[1])), target_list))) if target_list else ""
        input_record += "SEQUENCE_INCLUDED_REGION=%s\n" % (" ".join(map(lambda s: "%s,%s" % (str(s[0]), str(s[1])), included_region_list))) if included_region_list else ""
        input_record += "SEQUENCE_EXCLUDED_REGION=%s\n" % (" ".join(map(lambda s: "%s,%s" % (str(s[0]), str(s[1])), excluded_region_list))) if excluded_region_list else ""
        input_record += "SEQUENCE_INTERNAL_EXCLUDED_REGION=%s\n" % (" ".join(map(lambda s: "%s,%s" % (str(s[0]), str(s[1])), internal_oligo_excluded_region_list))) if internal_oligo_excluded_region_list else ""
        input_record += "SEQUENCE_OVERLAP_JUNCTION_LIST=%s\n" % (" ".join(map(str, overlap_junction_list))) if overlap_junction_list else ""

        input_record += "SEQUENCE_PRIMER=%s\n" % forward_primer if forward_primer else ""
        input_record += "SEQUENCE_PRIMER_REVCOMP=%s\n" % reverse_primer if reverse_primer else ""
        input_record += "SEQUENCE_INTERNAL_OLIGO=%s\n" % internal_oligo if internal_oligo else ""

        input_record += "SEQUENCE_FORCE_LEFT_START=%i\n" % force_forward_start if force_forward_start else ""
        input_record += "SEQUENCE_FORCE_LEFT_END=%i\n" % force_forward_end if force_forward_end else ""
        input_record += "SEQUENCE_FORCE_RIGHT_START=%i\n" % force_reverse_start if force_reverse_start else ""
        input_record += "SEQUENCE_FORCE_RIGHT_END=%i\n" % force_reverse_end if force_reverse_end else ""

        input_record += config
        input_record += "=\n"

        return input_record

    def predict_primers(self, input_file, output_file=None, settings_file=None, format_output=None, strict_tags=True,
                        error_file=None):
        options = self.parse_options(input_file,
                                     output_file=output_file,
                                     settings_file=settings_file,
                                     format_output=format_output,
                                     strict_tags=strict_tags,
                                     error_file=error_file)

        self.execute(options=options)


if __name__ == "__main__":
    pass
